@@ -12,7 +12,7 @@ extern int g_layer_below_window_level;
 static void window_did_receive_focus(struct window_manager *wm, struct mouse_state *ms, struct window *window)
 {
     struct window *focused_window = window_manager_find_window(wm, wm->focused_window_id);
-    if (focused_window && focused_window != window && window_space(focused_window) == window_space(window)) {
+    if (focused_window && focused_window != window && window_space(focused_window->id) == window_space(window->id)) {
         window_manager_set_window_opacity(wm, focused_window, g_window_manager.normal_window_opacity);
     }
 
@@ -56,25 +56,6 @@ static EVENT_HANDLER(APPLICATION_LAUNCHED)
         return;
     }
 
-    if (!workspace_application_is_observable(process)) {
-        debug("%s: %s (%d) is not observable, subscribing to activationPolicy changes\n", __FUNCTION__, process->name, process->pid);
-        workspace_application_observe_activation_policy(g_workspace_context, process);
-
-        //
-        // NOTE(koekeishiya): Do this again in case of race-conditions between the previous check and key-value observation subscription.
-        // Not actually sure if this can happen in practice..
-        //
-
-        if (workspace_application_is_observable(process)) {
-            @try {
-                NSRunningApplication *application = __atomic_load_n(&process->ns_application, __ATOMIC_RELAXED);
-                if (application && [application observationInfo]) {
-                    [application removeObserver:g_workspace_context forKeyPath:@"activationPolicy" context:process];
-                }
-            } @catch (NSException * __unused exception) {}
-        } else { return; }
-    }
-
     if (!workspace_application_is_finished_launching(process)) {
         debug("%s: %s (%d) is not finished launching, subscribing to finishedLaunching changes\n", __FUNCTION__, process->name, process->pid);
         workspace_application_observe_finished_launching(g_workspace_context, process);
@@ -94,6 +75,25 @@ static EVENT_HANDLER(APPLICATION_LAUNCHED)
         } else { return; }
     }
 
+    if (!workspace_application_is_observable(process)) {
+        debug("%s: %s (%d) is not observable, subscribing to activationPolicy changes\n", __FUNCTION__, process->name, process->pid);
+        workspace_application_observe_activation_policy(g_workspace_context, process);
+
+        //
+        // NOTE(koekeishiya): Do this again in case of race-conditions between the previous check and key-value observation subscription.
+        // Not actually sure if this can happen in practice..
+        //
+
+        if (workspace_application_is_observable(process)) {
+            @try {
+                NSRunningApplication *application = __atomic_load_n(&process->ns_application, __ATOMIC_RELAXED);
+                if (application && [application observationInfo]) {
+                    [application removeObserver:g_workspace_context forKeyPath:@"activationPolicy" context:process];
+                }
+            } @catch (NSException * __unused exception) {}
+        } else { return; }
+    }
+
     //
     // NOTE(koekeishiya): If we somehow receive a duplicate launched event due to the subscription-timing-mess above,
     // simply ignore the event..
@@ -151,7 +151,7 @@ static EVENT_HANDLER(APPLICATION_LAUNCHED)
         struct window *window = window_list[i];
 
         if (window_manager_should_manage_window(window) && !window_manager_find_managed_window(&g_window_manager, window)) {
-            if (default_origin) sid = window_space(window);
+            if (default_origin) sid = window_space(window->id);
 
             struct view *view = space_manager_find_view(&g_space_manager, sid);
             if (view->layout == VIEW_FLOAT) goto next;
@@ -170,7 +170,7 @@ static EVENT_HANDLER(APPLICATION_LAUNCHED)
             view_add_window_node_with_insertion_point(view, window, prev_window_id);
             window_manager_add_managed_window(&g_window_manager, window, view);
 
-            view->is_dirty = true;
+            view_set_flag(view, VIEW_IS_DIRTY);
             view_list[view_count++] = view;
 
             prev_window_id = window->id;
@@ -196,7 +196,8 @@ next:
         if (!space_is_visible(view->sid)) continue;
         if (!view_is_dirty(view))         continue;
 
-        view_flush(view);
+        window_node_flush(view->root);
+        view_clear_flag(view, VIEW_IS_DIRTY);
     }
 }
 
@@ -214,6 +215,13 @@ static EVENT_HANDLER(APPLICATION_TERMINATED)
     event_signal_push(SIGNAL_APPLICATION_TERMINATED, application);
     window_manager_remove_application(&g_window_manager, application->pid);
 
+    for (int i = 0; i < buf_len(g_window_manager.applications_to_refresh); ++i) {
+        if (application == g_window_manager.applications_to_refresh[i]) {
+            buf_del(g_window_manager.applications_to_refresh, i);
+            break;
+        }
+    }
+
     int window_count;
     struct window **window_list = window_manager_find_application_windows(&g_window_manager, application, &window_count);
 
@@ -222,7 +230,11 @@ static EVENT_HANDLER(APPLICATION_TERMINATED)
 
     for (int i = 0; i < window_count; ++i) {
         struct window *window = window_list[i];
-        __atomic_store_n(&window->id_ptr, NULL, __ATOMIC_RELEASE);
+
+        if (!__sync_bool_compare_and_swap(&window->id_ptr, &window->id, NULL)) {
+            window->application = NULL;
+            continue;
+        }
 
         struct view *view = window_manager_find_managed_window(&g_window_manager, window);
         if (view) {
@@ -240,7 +252,7 @@ static EVENT_HANDLER(APPLICATION_TERMINATED)
             view_remove_window_node(view, window);
             window_manager_remove_managed_window(&g_window_manager, window->id);
 
-            view->is_dirty = true;
+            view_set_flag(view, VIEW_IS_DIRTY);
             view_list[view_count++] = view;
         }
 
@@ -250,6 +262,7 @@ static EVENT_HANDLER(APPLICATION_TERMINATED)
             event_signal_push(SIGNAL_WINDOW_DESTROYED, window);
         }
 
+        window_manager_remove_scratchpad_for_window(&g_window_manager, window, false);
         window_manager_remove_window(&g_window_manager, window->id);
         window_unobserve(window);
         window_destroy(window);
@@ -272,7 +285,8 @@ static EVENT_HANDLER(APPLICATION_TERMINATED)
         if (!space_is_visible(view->sid)) continue;
         if (!view_is_dirty(view))         continue;
 
-        view_flush(view);
+        window_node_flush(view->root);
+        view_clear_flag(view, VIEW_IS_DIRTY);
     }
 
 out:
@@ -291,12 +305,24 @@ static EVENT_HANDLER(APPLICATION_FRONT_SWITCHED)
         return;
     }
 
+    struct application *deactivated_application = window_manager_find_application(&g_window_manager, g_process_manager.front_pid);
+    if (deactivated_application) event_signal_push(SIGNAL_APPLICATION_DEACTIVATED, deactivated_application);
+
     debug("%s: %s (%d)\n", __FUNCTION__, process->name, process->pid);
+    event_signal_push(SIGNAL_APPLICATION_ACTIVATED, application);
     g_process_manager.switch_event_time = GetCurrentEventTime();
     g_process_manager.last_front_pid = g_process_manager.front_pid;
     g_process_manager.front_pid = process->pid;
     event_signal_push(SIGNAL_APPLICATION_FRONT_SWITCHED, NULL);
 
+    for (int i = 0; i < buf_len(g_window_manager.applications_to_refresh); ++i) {
+        if (application == g_window_manager.applications_to_refresh[i]) {
+            debug("%s: %s has windows that are not yet resolved\n", __FUNCTION__, application->name);
+            window_manager_add_existing_application_windows(&g_space_manager, &g_window_manager, application, i);
+            break;
+        }
+    }
+
     uint32_t application_focused_window_id = application_focused_window(application);
     if (!application_focused_window_id) {
         struct window *focused_window = window_manager_find_window(&g_window_manager, g_window_manager.focused_window_id);
@@ -346,7 +372,7 @@ static EVENT_HANDLER(APPLICATION_VISIBLE)
         struct window *window = window_list[i];
 
         if (window_manager_should_manage_window(window) && !window_manager_find_managed_window(&g_window_manager, window)) {
-            struct view *view = space_manager_find_view(&g_space_manager, window_space(window));
+            struct view *view = space_manager_find_view(&g_space_manager, window_space(window->id));
             if (view->layout == VIEW_FLOAT) continue;
 
             //
@@ -363,7 +389,7 @@ static EVENT_HANDLER(APPLICATION_VISIBLE)
             view_add_window_node_with_insertion_point(view, window, prev_window_id);
             window_manager_add_managed_window(&g_window_manager, window, view);
 
-            view->is_dirty = true;
+            view_set_flag(view, VIEW_IS_DIRTY);
             view_list[view_count++] = view;
 
             prev_window_id = window->id;
@@ -384,7 +410,8 @@ static EVENT_HANDLER(APPLICATION_VISIBLE)
         if (!space_is_visible(view->sid)) continue;
         if (!view_is_dirty(view))         continue;
 
-        view_flush(view);
+        window_node_flush(view->root);
+        view_clear_flag(view, VIEW_IS_DIRTY);
     }
 
     event_signal_push(SIGNAL_APPLICATION_VISIBLE, application);
@@ -425,7 +452,7 @@ static EVENT_HANDLER(APPLICATION_HIDDEN)
             window_manager_remove_managed_window(&g_window_manager, window->id);
             window_manager_purify_window(&g_window_manager, window);
 
-            view->is_dirty = true;
+            view_set_flag(view, VIEW_IS_DIRTY);
             view_list[view_count++] = view;
         }
     }
@@ -444,7 +471,8 @@ static EVENT_HANDLER(APPLICATION_HIDDEN)
         if (!space_is_visible(view->sid)) continue;
         if (!view_is_dirty(view))         continue;
 
-        view_flush(view);
+        window_node_flush(view->root);
+        view_clear_flag(view, VIEW_IS_DIRTY);
     }
 
     event_signal_push(SIGNAL_APPLICATION_HIDDEN, application);
@@ -464,14 +492,25 @@ static EVENT_HANDLER(WINDOW_CREATED)
     struct application *application = window_manager_find_application(&g_window_manager, window_pid);
     if (!application) { CFRelease(context); return; }
 
-    struct window *window = window_manager_create_and_add_window(&g_space_manager, &g_window_manager, application, context, window_id);
+    struct window *window = window_manager_create_and_add_window(&g_space_manager, &g_window_manager, application, context, window_id, true);
     if (!window) return;
 
+    int rule_len = buf_len(g_window_manager.rules);
+    for (int i = 0; i < rule_len; ++i) {
+        if (rule_check_flag(&g_window_manager.rules[i], RULE_ONE_SHOT_REMOVE)) {
+            rule_destroy(&g_window_manager.rules[i]);
+            if (buf_del(g_window_manager.rules, i)) {
+                --i;
+                --rule_len;
+            }
+        }
+    }
+
     if (window_manager_should_manage_window(window) && !window_manager_find_managed_window(&g_window_manager, window)) {
         uint64_t sid;
 
         if (g_window_manager.window_origin_mode == WINDOW_ORIGIN_DEFAULT) {
-            sid = window_space(window);
+            sid = window_space(window->id);
         } else if (g_window_manager.window_origin_mode == WINDOW_ORIGIN_FOCUSED) {
             sid = g_space_manager.current_space_id;
         } else /* if (g_window_manager.window_origin_mode == WINDOW_ORIGIN_CURSOR) */ {
@@ -495,7 +534,7 @@ static EVENT_HANDLER(WINDOW_DESTROYED)
         return;
     }
 
-    debug("%s: %s %d\n", __FUNCTION__, window->application->name, window->id);
+    debug("%s: %s %d\n", __FUNCTION__, window->application ? window->application->name : "<unknown>", window->id);
 
     struct view *view = window_manager_find_managed_window(&g_window_manager, window);
     if (view) {
@@ -509,6 +548,7 @@ static EVENT_HANDLER(WINDOW_DESTROYED)
         event_signal_push(SIGNAL_WINDOW_DESTROYED, window);
     }
 
+    window_manager_remove_scratchpad_for_window(&g_window_manager, window, false);
     window_manager_remove_window(&g_window_manager, window->id);
     window_destroy(window);
 }
@@ -567,6 +607,24 @@ static EVENT_HANDLER(WINDOW_MOVED)
     debug("%s: %s %d\n", __FUNCTION__, window->application->name, window->id);
     event_signal_push(SIGNAL_WINDOW_MOVED, window);
     window->frame.origin = new_origin;
+
+    if (!g_mouse_state.window || g_mouse_state.window != window) {
+        struct view *view = window_manager_find_managed_window(&g_window_manager, window);
+        if (view) {
+            struct window_node *node = view_find_window_node(view, window->id);
+            if (node && (AX_DIFF(node->area.x, new_origin.x) ||
+                         AX_DIFF(node->area.y, new_origin.y))
+                     &&
+               (!node->zoom || AX_DIFF(node->zoom->area.x, new_origin.x) ||
+                               AX_DIFF(node->zoom->area.y, new_origin.y))) {
+                if (space_is_visible(view->sid)) {
+                    window_node_flush(node);
+                } else {
+                    view_set_flag(view, VIEW_IS_DIRTY);
+                }
+            }
+        }
+    }
 }
 
 static EVENT_HANDLER(WINDOW_RESIZED)
@@ -616,13 +674,35 @@ static EVENT_HANDLER(WINDOW_RESIZED)
         window_manager_wait_for_native_fullscreen_transition(window);
 
         if (window_manager_should_manage_window(window) && !window_manager_find_managed_window(&g_window_manager, window)) {
-            struct view *view = space_manager_tile_window_on_space(&g_space_manager, window, window_space(window));
+            struct view *view = space_manager_tile_window_on_space(&g_space_manager, window, window_space(window->id));
             window_manager_add_managed_window(&g_window_manager, window, view);
         }
     } else if (!was_fullscreen == !is_fullscreen) {
         if (g_mouse_state.current_action == MOUSE_MODE_MOVE && g_mouse_state.window == window) {
             g_mouse_state.window_frame.size = g_mouse_state.window->frame.size;
         }
+
+        if (!g_mouse_state.window || g_mouse_state.window != window) {
+            struct view *view = window_manager_find_managed_window(&g_window_manager, window);
+            if (view) {
+                struct window_node *node = view_find_window_node(view, window->id);
+                if (node && (AX_DIFF(node->area.x, new_frame.origin.x)   ||
+                             AX_DIFF(node->area.y, new_frame.origin.y)   ||
+                             AX_DIFF(node->area.w, new_frame.size.width) ||
+                             AX_DIFF(node->area.h, new_frame.size.height))
+                         &&
+                   (!node->zoom || AX_DIFF(node->zoom->area.x, new_frame.origin.x)   ||
+                                   AX_DIFF(node->zoom->area.y, new_frame.origin.y)   ||
+                                   AX_DIFF(node->zoom->area.w, new_frame.size.width) ||
+                                   AX_DIFF(node->zoom->area.h, new_frame.size.height))) {
+                    if (space_is_visible(view->sid)) {
+                        window_node_flush(node);
+                    } else {
+                        view_set_flag(view, VIEW_IS_DIRTY);
+                    }
+                }
+            }
+        }
     }
 }
 
@@ -700,63 +780,12 @@ static EVENT_HANDLER(WINDOW_TITLE_CHANGED)
     event_signal_push(SIGNAL_WINDOW_TITLE_CHANGED, window);
 }
 
-static EVENT_HANDLER(SLS_SPACE_ADD_WINDOW)
+static EVENT_HANDLER(SLS_WINDOW_ORDERED)
 {
-    uint32_t window_id = (uint32_t) param1;
-    struct window *window = window_manager_find_window(&g_window_manager, window_id);
-    if (!window) return;
-
-    if (!__sync_bool_compare_and_swap(&window->id_ptr, &window->id, &window->id)) {
-        debug("%s: %d has been marked invalid by the system, ignoring event..\n", __FUNCTION__, window_id);
-        return;
-    }
-
-    uint64_t sid = (uint64_t)(intptr_t) context;
-    debug("%s: %d added to %lld\n", __FUNCTION__, window_id, sid);
-
-    if (window_manager_should_manage_window(window) && !window_manager_find_managed_window(&g_window_manager, window)) {
-        struct view *view = space_manager_tile_window_on_space(&g_space_manager, window, sid);
-        window_manager_add_managed_window(&g_window_manager, window, view);
-    }
-
-    // event_signal_push(SIGNAL_SPACE_ADD_WINDOW, context);
-}
-
-static EVENT_HANDLER(SLS_SPACE_REMOVE_WINDOW)
-{
-    uint32_t window_id = (uint32_t) param1;
-    struct window *window = window_manager_find_window(&g_window_manager, window_id);
-    if (!window) return;
-
-    if (!__sync_bool_compare_and_swap(&window->id_ptr, &window->id, &window->id)) {
-        debug("%s: %d has been marked invalid by the system, ignoring event..\n", __FUNCTION__, window_id);
-        return;
-    }
-
-    uint64_t sid = (uint64_t)(intptr_t) context;
-    debug("%s: %d removed from %lld\n", __FUNCTION__, window_id, sid);
-
-    struct view *view = window_manager_find_managed_window(&g_window_manager, window);
-    if (view) {
-        window_manager_adjust_layer(window, LAYER_NORMAL);
-        struct window_node *node = view_remove_window_node(view, window);
-        window_manager_remove_managed_window(&g_window_manager, window->id);
-
-        if (node) {
-            if (space_is_visible(view->sid)) {
-                window_node_flush(node);
-            } else {
-                view->is_dirty = true;
-            }
-        }
-    }
-
-    if (window_manager_should_manage_window(window)) {
-        struct view *view = space_manager_tile_window_on_space(&g_space_manager, window, window_space(window));
-        window_manager_add_managed_window(&g_window_manager, window, view);
-    }
-
-    // event_signal_push(SIGNAL_SPACE_REMOVE_WINDOW, context);
+    uint32_t wid = (uint64_t)(intptr_t) context;
+    debug("%s: %d\n", __FUNCTION__, wid);
+    struct window_node *node = table_find(&g_window_manager.insert_feedback, &wid);
+    if (node) SLSOrderWindow(g_connection, node->feedback_window.id, 1, node->window_order[0]);
 }
 
 static EVENT_HANDLER(SLS_SPACE_CREATED)
@@ -770,9 +799,7 @@ static EVENT_HANDLER(SLS_SPACE_DESTROYED)
 {
     uint64_t sid = (uint64_t)(intptr_t) context;
     debug("%s: %lld\n", __FUNCTION__, sid);
-
     space_manager_remove_label_for_space(&g_space_manager, sid);
-
     event_signal_push(SIGNAL_SPACE_DESTROYED, context);
 }
 
@@ -781,6 +808,11 @@ static EVENT_HANDLER(SPACE_CHANGED)
     g_space_manager.last_space_id = g_space_manager.current_space_id;
     g_space_manager.current_space_id = space_manager_active_space();
 
+    if (g_window_manager.menubar_opacity != 1.0f) {
+        float alpha = space_is_fullscreen(g_space_manager.current_space_id) ? 1.0f : g_window_manager.menubar_opacity;
+        SLSSetMenuBarInsetAndAlpha(g_connection, 0, 1, alpha);
+    }
+
     debug("%s: %lld\n", __FUNCTION__, g_space_manager.current_space_id);
     struct view *view = space_manager_find_view(&g_space_manager, g_space_manager.current_space_id);
 
@@ -795,8 +827,14 @@ static EVENT_HANDLER(SPACE_CHANGED)
     if (!mission_control_is_active() && space_is_user(g_space_manager.current_space_id)) {
         window_manager_validate_and_check_for_windows_on_space(&g_space_manager, &g_window_manager, g_space_manager.current_space_id);
 
-        if (view_is_invalid(view)) view_update(view);
-        if (view_is_dirty(view))   view_flush(view);
+        if (view_is_invalid(view)) {
+            view_update(view);
+        }
+
+        if (view_is_dirty(view)) {
+            window_node_flush(view->root);
+            view_clear_flag(view, VIEW_IS_DIRTY);
+        }
     }
 
     event_signal_push(SIGNAL_SPACE_CHANGED, NULL);
@@ -822,6 +860,11 @@ static EVENT_HANDLER(DISPLAY_CHANGED)
         return;
     }
 
+    if (g_window_manager.menubar_opacity != 1.0f) {
+        float alpha = space_is_fullscreen(g_space_manager.current_space_id) ? 1.0f : g_window_manager.menubar_opacity;
+        SLSSetMenuBarInsetAndAlpha(g_connection, 0, 1, alpha);
+    }
+
     debug("%s: %d %lld\n", __FUNCTION__, g_display_manager.current_display_id, g_space_manager.current_space_id);
     struct view *view = space_manager_find_view(&g_space_manager, g_space_manager.current_space_id);
 
@@ -836,8 +879,14 @@ static EVENT_HANDLER(DISPLAY_CHANGED)
     if (!mission_control_is_active() && space_is_user(g_space_manager.current_space_id)) {
         window_manager_validate_and_check_for_windows_on_space(&g_space_manager, &g_window_manager, g_space_manager.current_space_id);
 
-        if (view_is_invalid(view)) view_update(view);
-        if (view_is_dirty(view))   view_flush(view);
+        if (view_is_invalid(view)) {
+            view_update(view);
+        }
+
+        if (view_is_dirty(view)) {
+            window_node_flush(view->root);
+            view_clear_flag(view, VIEW_IS_DIRTY);
+        }
     }
 
     event_signal_push(SIGNAL_DISPLAY_CHANGED, NULL);
@@ -854,9 +903,10 @@ static EVENT_HANDLER(DISPLAY_ADDED)
 
 static EVENT_HANDLER(DISPLAY_REMOVED)
 {
-    uint32_t did = display_manager_main_display_id();
+    uint32_t did = (uint32_t)(intptr_t) context;
     debug("%s: %d\n", __FUNCTION__, did);
-    window_manager_handle_display_add_and_remove(&g_space_manager, &g_window_manager, did);
+    display_manager_remove_label_for_display(&g_display_manager, did);
+    window_manager_handle_display_add_and_remove(&g_space_manager, &g_window_manager, display_manager_main_display_id());
     event_signal_push(SIGNAL_DISPLAY_REMOVED, context);
 }
 
@@ -882,15 +932,15 @@ static EVENT_HANDLER(MOUSE_DOWN)
     if (g_mouse_state.current_action != MOUSE_MODE_NONE) goto out;
 
     CGPoint point = CGEventGetLocation(context);
-    uint32_t wid = CGEventGetIntegerValueField(context, kCGMouseEventWindowUnderMousePointer);
-    debug("%s: %d %.2f, %.2f\n", __FUNCTION__, wid, point.x, point.y);
+    debug("%s: %.2f, %.2f\n", __FUNCTION__, point.x, point.y);
 
-    struct window *window = wid ? window_manager_find_window(&g_window_manager, wid) : window_manager_find_window_at_point(&g_window_manager, point);
+    struct window *window = window_manager_find_window_at_point(&g_window_manager, point);
     if (!window || window_is_fullscreen(window)) goto out;
 
     g_mouse_state.window = window;
-    g_mouse_state.down_location = point;
     g_mouse_state.window_frame = g_mouse_state.window->frame;
+    g_mouse_state.down_location = point;
+    g_mouse_state.direction = 0;
 
     int64_t button = CGEventGetIntegerValueField(context, kCGMouseEventButtonNumber);
     uint8_t mod = (uint8_t) param1;
@@ -901,6 +951,14 @@ static EVENT_HANDLER(MOUSE_DOWN)
         g_mouse_state.current_action = g_mouse_state.action2;
     }
 
+    if (g_mouse_state.current_action == MOUSE_MODE_RESIZE) {
+        CGPoint frame_mid = { CGRectGetMidX(g_mouse_state.window_frame), CGRectGetMidY(g_mouse_state.window_frame) };
+        if (point.x < frame_mid.x) g_mouse_state.direction |= HANDLE_LEFT;
+        if (point.y < frame_mid.y) g_mouse_state.direction |= HANDLE_TOP;
+        if (point.x > frame_mid.x) g_mouse_state.direction |= HANDLE_RIGHT;
+        if (point.y > frame_mid.y) g_mouse_state.direction |= HANDLE_BOTTOM;
+    }
+
 out:
     CFRelease(context);
 }
@@ -908,7 +966,7 @@ out:
 static EVENT_HANDLER(MOUSE_UP)
 {
     if (mission_control_is_active()) goto out;
-    if (!g_mouse_state.window)       goto out;
+    if (!g_mouse_state.window)       goto res;
 
     if (!__sync_bool_compare_and_swap(&g_mouse_state.window->id_ptr, &g_mouse_state.window->id, &g_mouse_state.window->id)) {
         debug("%s: %d has been marked invalid by the system, ignoring event..\n", __FUNCTION__, g_mouse_state.window->id);
@@ -941,6 +999,12 @@ static EVENT_HANDLER(MOUSE_UP)
         struct window_node *b_node = window ? view_find_window_node(dst_view, window->id) : NULL;
 
         if (a_node && b_node && a_node != b_node) {
+            if (g_mouse_state.feedback_node) {
+                g_mouse_state.feedback_node->insert_dir = 0;
+                insert_feedback_destroy(g_mouse_state.feedback_node);
+                g_mouse_state.feedback_node = NULL;
+            }
+
             enum mouse_drop_action drop_action = mouse_determine_drop_action(&g_mouse_state, a_node, window, point);
             switch (drop_action) {
             case MOUSE_DROP_ACTION_STACK: {
@@ -974,8 +1038,8 @@ static EVENT_HANDLER(MOUSE_UP)
 
 err:
     g_mouse_state.window = NULL;
+res:
     g_mouse_state.current_action = MOUSE_MODE_NONE;
-
 out:
     CFRelease(context);
 }
@@ -1010,28 +1074,82 @@ static EVENT_HANDLER(MOUSE_DRAGGED)
             window_manager_move_window(g_mouse_state.window, new_point.x, new_point.y);
         }
     } else if (g_mouse_state.current_action == MOUSE_MODE_RESIZE) {
-        uint64_t event_time = CGEventGetTimestamp(context);
-        float dt = ((float) event_time - g_mouse_state.last_moved_time) * (1.0f / 1E6);
-        if (dt < 66.67f) goto out;
+        uint64_t event_time = read_os_timer();
+        float dt = ((float) event_time - g_mouse_state.last_moved_time) * (1000.0f / (float)read_os_freq());
+        if (dt < 67.67f) goto out;
 
         int dx = point.x - g_mouse_state.down_location.x;
         int dy = point.y - g_mouse_state.down_location.y;
 
-        uint8_t direction = 0;
-        CGRect frame = g_mouse_state.window->frame;
-        CGPoint frame_mid = { CGRectGetMidX(frame), CGRectGetMidY(frame) };
-
-        if (point.x < frame_mid.x) direction |= HANDLE_LEFT;
-        if (point.y < frame_mid.y) direction |= HANDLE_TOP;
-        if (point.x > frame_mid.x) direction |= HANDLE_RIGHT;
-        if (point.y > frame_mid.y) direction |= HANDLE_BOTTOM;
-
-        window_manager_resize_window_relative_internal(g_mouse_state.window, frame, direction, dx, dy, false);
+        window_manager_resize_window_relative_internal(g_mouse_state.window, g_mouse_state.window->frame, g_mouse_state.direction, dx, dy, false);
 
         g_mouse_state.last_moved_time = event_time;
         g_mouse_state.down_location = point;
     }
 
+    struct view *src_view = window_manager_find_managed_window(&g_window_manager, g_mouse_state.window);
+    if (!src_view) goto out;
+
+    struct mouse_window_info info;
+    mouse_window_info_populate(&g_mouse_state, &info);
+
+    if (info.changed_position && !info.changed_size) {
+        uint64_t cursor_sid = display_space_id(display_manager_point_display_id(point));
+        struct view *dst_view = space_manager_find_view(&g_space_manager, cursor_sid);
+
+        struct window *window = window_manager_find_window_at_point_filtering_window(&g_window_manager, point, g_mouse_state.window->id);
+        if (!window) window = window_manager_find_window_at_point(&g_window_manager, point);
+        if (window == g_mouse_state.window) window = NULL;
+
+        struct window_node *a_node = view_find_window_node(src_view, g_mouse_state.window->id);
+        struct window_node *b_node = window ? view_find_window_node(dst_view, window->id) : NULL;
+
+        if (a_node && b_node && a_node != b_node) {
+            if (g_mouse_state.feedback_node && g_mouse_state.feedback_node != b_node) {
+                g_mouse_state.feedback_node->insert_dir = 0;
+                insert_feedback_destroy(g_mouse_state.feedback_node);
+            }
+
+            int insert_dir = 0;
+            enum mouse_drop_action drop_action = mouse_determine_drop_action(&g_mouse_state, a_node, window, point);
+            switch (drop_action) {
+            case MOUSE_DROP_ACTION_STACK: {
+                insert_dir = STACK;
+            } break;
+            case MOUSE_DROP_ACTION_SWAP: {
+                insert_dir = STACK;
+            } break;
+            case MOUSE_DROP_ACTION_WARP_TOP: {
+                insert_dir = DIR_NORTH;
+            } break;
+            case MOUSE_DROP_ACTION_WARP_RIGHT: {
+                insert_dir = DIR_EAST;
+            } break;
+            case MOUSE_DROP_ACTION_WARP_BOTTOM: {
+                insert_dir = DIR_SOUTH;
+            } break;
+            case MOUSE_DROP_ACTION_WARP_LEFT: {
+                insert_dir = DIR_WEST;
+            } break;
+            case MOUSE_DROP_ACTION_NONE: {
+                /* silence compiler warning.. */
+            } break;
+            }
+
+            if (b_node->insert_dir != insert_dir) {
+                b_node->insert_dir = insert_dir;
+                insert_feedback_show(b_node);
+                g_mouse_state.feedback_node = b_node;
+            }
+        } else if (!b_node) {
+            if (g_mouse_state.feedback_node) {
+                g_mouse_state.feedback_node->insert_dir = 0;
+                insert_feedback_destroy(g_mouse_state.feedback_node);
+                g_mouse_state.feedback_node = NULL;
+            }
+        }
+    }
+
 out:
     CFRelease(context);
 }
@@ -1042,15 +1160,9 @@ static EVENT_HANDLER(MOUSE_MOVED)
     if (mission_control_is_active())               goto out;
     if (g_mouse_state.ffm_window_id)               goto out;
 
-    uint64_t event_time = CGEventGetTimestamp(context);
-    float dt = ((float) event_time - g_mouse_state.last_moved_time) * (1.0f / 1E6);
-    if (dt < 25.0f) goto out;
-
-    uint32_t wid = CGEventGetIntegerValueField(context, kCGMouseEventWindowUnderMousePointer);
     CGPoint point = CGEventGetLocation(context);
-    g_mouse_state.last_moved_time = event_time;
+    struct window *window = window_manager_find_window_at_point(&g_window_manager, point);
 
-    struct window *window = wid ? window_manager_find_window(&g_window_manager, wid) : window_manager_find_window_at_point(&g_window_manager, point);
     if (window) {
         if (window->id == g_window_manager.focused_window_id) goto out;
         if (!window_manager_is_window_eligible(window))       goto out;
@@ -1129,10 +1241,10 @@ static EVENT_HANDLER(MOUSE_MOVED)
         uint32_t cursor_did = display_manager_point_display_id(point);
         if (g_display_manager.current_display_id == cursor_did) goto out;
 
-        CGRect menu = display_manager_menu_bar_rect(cursor_did);
-        if (cgrect_contains_point(menu, point)) goto out;
+        CGRect bounds = display_bounds_constrained(cursor_did);
+        if (!cgrect_contains_point(bounds, point)) goto out;
 
-        int wid = display_manager_focus_display_with_point(cursor_did, point, false);
+        uint32_t wid = display_manager_focus_display_with_point(cursor_did, point, false);
         g_mouse_state.ffm_window_id = wid;
     }
 
@@ -1144,12 +1256,6 @@ static EVENT_HANDLER(MISSION_CONTROL_SHOW_ALL_WINDOWS)
 {
     debug("%s:\n", __FUNCTION__);
     g_mission_control_mode = MISSION_CONTROL_MODE_SHOW_ALL_WINDOWS;
-
-    for (int i = 0; i < buf_len(g_window_manager.insert_feedback_windows); ++i) {
-        uint32_t feedback_wid = g_window_manager.insert_feedback_windows[i];
-        SLSOrderWindow(g_connection, feedback_wid, 0, 0);
-    }
-
     event_signal_push(SIGNAL_MISSION_CONTROL_ENTER, (void*)(uintptr_t)g_mission_control_mode);
 }
 
@@ -1157,12 +1263,6 @@ static EVENT_HANDLER(MISSION_CONTROL_SHOW_FRONT_WINDOWS)
 {
     debug("%s:\n", __FUNCTION__);
     g_mission_control_mode = MISSION_CONTROL_MODE_SHOW_FRONT_WINDOWS;
-
-    for (int i = 0; i < buf_len(g_window_manager.insert_feedback_windows); ++i) {
-        uint32_t feedback_wid = g_window_manager.insert_feedback_windows[i];
-        SLSOrderWindow(g_connection, feedback_wid, 0, 0);
-    }
-
     event_signal_push(SIGNAL_MISSION_CONTROL_ENTER, (void*)(uintptr_t)g_mission_control_mode);
 }
 
@@ -1170,12 +1270,6 @@ static EVENT_HANDLER(MISSION_CONTROL_SHOW_DESKTOP)
 {
     debug("%s:\n", __FUNCTION__);
     g_mission_control_mode = MISSION_CONTROL_MODE_SHOW_DESKTOP;
-
-    for (int i = 0; i < buf_len(g_window_manager.insert_feedback_windows); ++i) {
-        uint32_t feedback_wid = g_window_manager.insert_feedback_windows[i];
-        SLSOrderWindow(g_connection, feedback_wid, 0, 0);
-    }
-
     event_signal_push(SIGNAL_MISSION_CONTROL_ENTER, (void*)(uintptr_t)g_mission_control_mode);
 }
 
@@ -1184,11 +1278,6 @@ static EVENT_HANDLER(MISSION_CONTROL_ENTER)
     debug("%s:\n", __FUNCTION__);
     g_mission_control_mode = MISSION_CONTROL_MODE_SHOW;
 
-    for (int i = 0; i < buf_len(g_window_manager.insert_feedback_windows); ++i) {
-        uint32_t feedback_wid = g_window_manager.insert_feedback_windows[i];
-        SLSOrderWindow(g_connection, feedback_wid, 0, 0);
-    }
-
     dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 0.1f * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
         event_loop_post(&g_event_loop, MISSION_CONTROL_CHECK_FOR_EXIT, NULL, 0);
     });
@@ -1243,15 +1332,13 @@ static EVENT_HANDLER(MISSION_CONTROL_EXIT)
 {
     debug("%s:\n", __FUNCTION__);
 
-    for (int i = 0; i < buf_len(g_window_manager.insert_feedback_windows); ++i) {
-        uint32_t feedback_wid = g_window_manager.insert_feedback_windows[i];
-        SLSOrderWindow(g_connection, feedback_wid, 1, 0);
+    if (g_window_manager.menubar_opacity != 1.0f) {
+        float alpha = space_is_fullscreen(g_space_manager.current_space_id) ? 1.0f : g_window_manager.menubar_opacity;
+        SLSSetMenuBarInsetAndAlpha(g_connection, 0, 1, alpha);
     }
 
-    if (!workspace_is_macos_ventura() && !workspace_is_macos_sonoma()) {
-        if (g_mission_control_mode == MISSION_CONTROL_MODE_SHOW || g_mission_control_mode == MISSION_CONTROL_MODE_SHOW_ALL_WINDOWS) {
-            window_manager_correct_for_mission_control_changes(&g_space_manager, &g_window_manager);
-        }
+    if (g_mission_control_mode == MISSION_CONTROL_MODE_SHOW || g_mission_control_mode == MISSION_CONTROL_MODE_SHOW_ALL_WINDOWS) {
+        window_manager_correct_for_mission_control_changes(&g_space_manager, &g_window_manager);
     }
 
     event_signal_push(SIGNAL_MISSION_CONTROL_EXIT, (void*)(uintptr_t)g_mission_control_mode);
@@ -1276,8 +1363,9 @@ static int is_menu_open = 0;
 static EVENT_HANDLER(MENU_OPENED)
 {
     debug("%s\n", __FUNCTION__);
+    ++is_menu_open;
 
-    if (++is_menu_open == 1) {
+    if (is_menu_open == 1) {
         ffm_value = g_window_manager.ffm_mode;
         g_window_manager.ffm_mode = FFM_DISABLED;
     }
@@ -1286,8 +1374,13 @@ static EVENT_HANDLER(MENU_OPENED)
 static EVENT_HANDLER(MENU_CLOSED)
 {
     debug("%s\n", __FUNCTION__);
+    --is_menu_open;
 
-    if (--is_menu_open == 0) {
+    if (is_menu_open < 0) {
+        is_menu_open = 0;
+    }
+
+    if (is_menu_open == 0) {
         g_window_manager.ffm_mode = ffm_value;
     }
 }
@@ -1315,10 +1408,14 @@ static EVENT_HANDLER(SYSTEM_WOKE)
         window_manager_set_window_opacity(&g_window_manager, focused_window, g_window_manager.active_window_opacity);
         window_manager_center_mouse(&g_window_manager, focused_window);
     }
+
+    event_signal_push(SIGNAL_SYSTEM_WOKE, NULL);
 }
 
 static EVENT_HANDLER(DAEMON_MESSAGE)
 {
+    TIME_FUNCTION;
+
     FILE *rsp         = NULL;
     int bytes_read    = 0;
     int bytes_to_read = 0;
@@ -1356,6 +1453,8 @@ static void *event_loop_run(void *context)
         NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
 
         for (;;) {
+            profile_begin();
+
             do {
                 head = __atomic_load_n(&event_loop->head, __ATOMIC_RELAXED);
                 next = __atomic_load_n(&head->next, __ATOMIC_RELAXED);
@@ -1370,6 +1469,8 @@ static void *event_loop_run(void *context)
 
             event_signal_flush();
             ts_reset();
+
+            profile_end_and_print();
         }
 
 empty:
