@@ -1,5 +1,4 @@
 extern struct event_loop g_event_loop;
-extern pid_t g_pid;
 
 static inline uint8_t mouse_mod_from_cgflags(uint32_t cgflags)
 {
@@ -16,6 +15,7 @@ static inline uint8_t mouse_mod_from_cgflags(uint32_t cgflags)
 
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wswitch"
+#pragma clang diagnostic ignored "-Wdeprecated-declarations"
 static MOUSE_HANDLER(mouse_handler)
 {
     struct mouse_state *mouse_state = context;
@@ -28,43 +28,34 @@ static MOUSE_HANDLER(mouse_handler)
     } break;
     case kCGEventLeftMouseDown:
     case kCGEventRightMouseDown: {
-
-        //
-        // :SynthesizedEvent
-        //
-        // NOTE(koekeishiya): This event-tap is placed at the "Annotated Session" location, which
-        // causes yabai to also intercept events that we post (notably, the way in which window
-        // focusing between separate applications have been implemented). Skip these events...
-        //
-
-        pid_t source_pid = CGEventGetIntegerValueField(event, kCGEventSourceUnixProcessID);
-        if (source_pid == g_pid) return event;
-
         uint8_t mod = mouse_mod_from_cgflags(CGEventGetFlags(event));
         event_loop_post(&g_event_loop, MOUSE_DOWN, (void *) CFRetain(event), mod);
 
-        mouse_state->consume_mouse_click = mod == mouse_state->modifier;
-        if (mouse_state->consume_mouse_click) return NULL;
+        if (mod == mouse_state->modifier) {
+            mouse_state->consume_mouse_click = true;
+            mouse_state->consumed_event = (CGEventRef) CFRetain(event);
+            return NULL;
+        }
     } break;
     case kCGEventLeftMouseUp:
     case kCGEventRightMouseUp: {
-
-        //
-        // :SynthesizedEvent
-        //
-        // NOTE(koekeishiya): This event-tap is placed at the "Annotated Session" location, which
-        // causes yabai to also intercept events that we post (notably, the way in which window
-        // focusing between separate applications have been implemented). Skip these events...
-        //
-
-        pid_t source_pid = CGEventGetIntegerValueField(event, kCGEventSourceUnixProcessID);
-        if (source_pid == g_pid) return event;
-
         event_loop_post(&g_event_loop, MOUSE_UP, (void *) CFRetain(event), 0);
-        if (mouse_state->consume_mouse_click) return NULL;
+
+        if (mouse_state->consume_mouse_click) {
+            if (!mouse_state->drag_detected) {
+                CGEventTapPostEvent(proxy, mouse_state->consumed_event);
+                CGEventTapPostEvent(proxy, event);
+            }
+
+            mouse_state->drag_detected = false;
+            mouse_state->consume_mouse_click = false;
+            CFRelease(mouse_state->consumed_event);
+            return NULL;
+        }
     } break;
     case kCGEventLeftMouseDragged:
     case kCGEventRightMouseDragged: {
+        mouse_state->drag_detected = true;
         event_loop_post(&g_event_loop, MOUSE_DRAGGED, (void *) CFRetain(event), 0);
     } break;
     case kCGEventMouseMoved: {
@@ -267,7 +258,7 @@ bool mouse_handler_begin(struct mouse_state *mouse_state, uint32_t mask)
 {
     if (mouse_state->handle) return true;
 
-    mouse_state->handle = CGEventTapCreate(kCGAnnotatedSessionEventTap, kCGHeadInsertEventTap, kCGEventTapOptionDefault, mask, mouse_handler, mouse_state);
+    mouse_state->handle = CGEventTapCreate(kCGHIDEventTap, kCGHeadInsertEventTap, kCGEventTapOptionDefault, mask, mouse_handler, mouse_state);
     if (!mouse_state->handle) return false;
 
     if (!CGEventTapIsEnabled(mouse_state->handle)) {
