@@ -11,20 +11,24 @@ static TABLE_COMPARE_FUNC(compare_view)
     return *(uint64_t *) key_a == *(uint64_t *) key_b;
 }
 
-bool space_manager_query_space(FILE *rsp, uint64_t sid)
+bool space_manager_query_space(FILE *rsp, uint64_t sid, uint64_t flags)
 {
+    TIME_FUNCTION;
+
     struct view *view = space_manager_query_view(&g_space_manager, sid);
     if (!view) return false;
 
-    view_serialize(rsp, view);
+    view_serialize(rsp, view, flags);
     fprintf(rsp, "\n");
     return true;
 }
 
-bool space_manager_query_spaces_for_window(FILE *rsp, struct window *window)
+bool space_manager_query_spaces_for_window(FILE *rsp, struct window *window, uint64_t flags)
 {
+    TIME_FUNCTION;
+
     int space_count;
-    uint64_t *space_list = window_space_list(window, &space_count);
+    uint64_t *space_list = window_space_list(window->id, &space_count);
     if (!space_list) return false;
 
     fprintf(rsp, "[");
@@ -32,7 +36,7 @@ bool space_manager_query_spaces_for_window(FILE *rsp, struct window *window)
         struct view *view = space_manager_query_view(&g_space_manager, space_list[i]);
         if (!view) continue;
 
-        view_serialize(rsp, view);
+        view_serialize(rsp, view, flags);
         fprintf(rsp, "%c", i < space_count - 1 ? ',' : ']');
     }
     fprintf(rsp, "\n");
@@ -40,8 +44,10 @@ bool space_manager_query_spaces_for_window(FILE *rsp, struct window *window)
     return true;
 }
 
-bool space_manager_query_spaces_for_display(FILE *rsp, uint32_t did)
+bool space_manager_query_spaces_for_display(FILE *rsp, uint32_t did, uint64_t flags)
 {
+    TIME_FUNCTION;
+
     int space_count;
     uint64_t *space_list = display_space_list(did, &space_count);
     if (!space_list) return false;
@@ -51,7 +57,7 @@ bool space_manager_query_spaces_for_display(FILE *rsp, uint32_t did)
         struct view *view = space_manager_query_view(&g_space_manager, space_list[i]);
         if (!view) continue;
 
-        view_serialize(rsp, view);
+        view_serialize(rsp, view, flags);
         fprintf(rsp, "%c", i < space_count - 1 ? ',' : ']');
     }
     fprintf(rsp, "\n");
@@ -59,8 +65,10 @@ bool space_manager_query_spaces_for_display(FILE *rsp, uint32_t did)
     return true;
 }
 
-bool space_manager_query_spaces_for_displays(FILE *rsp)
+bool space_manager_query_spaces_for_displays(FILE *rsp, uint64_t flags)
 {
+    TIME_FUNCTION;
+
     int display_count;
     uint32_t *display_list = display_manager_active_display_list(&display_count);
     if (!display_list) return false;
@@ -75,7 +83,7 @@ bool space_manager_query_spaces_for_displays(FILE *rsp)
             struct view *view = space_manager_query_view(&g_space_manager, space_list[j]);
             if (!view) continue;
 
-            view_serialize(rsp, view);
+            view_serialize(rsp, view, flags);
             if (j < space_count - 1) fprintf(rsp, ",");
         }
 
@@ -116,7 +124,7 @@ void space_manager_mark_view_invalid(struct space_manager *sm,  uint64_t sid)
     struct view *view = space_manager_find_view(sm, sid);
     if (view->layout == VIEW_FLOAT) return;
 
-    view->is_valid = false;
+    view_clear_flag(view, VIEW_IS_VALID);
 }
 
 void space_manager_untile_window(struct space_manager *sm, struct view *view, struct window *window)
@@ -130,7 +138,7 @@ void space_manager_untile_window(struct space_manager *sm, struct view *view, st
     if (space_is_visible(view->sid)) {
         window_node_flush(node);
     } else {
-        view->is_dirty = true;
+        view_set_flag(view, VIEW_IS_DIRTY);
     }
 }
 
@@ -224,7 +232,12 @@ bool space_manager_toggle_gap_for_space(struct space_manager *sm, uint64_t sid)
     struct view *view = space_manager_find_view(sm, sid);
     if (view->layout == VIEW_FLOAT) return false;
 
-    view->enable_gap = !view->enable_gap;
+    if (view_check_flag(view, VIEW_ENABLE_GAP)) {
+        view_clear_flag(view, VIEW_ENABLE_GAP);
+    } else {
+        view_set_flag(view, VIEW_ENABLE_GAP);
+    }
+
     view_update(view);
     view_flush(view);
 
@@ -251,7 +264,7 @@ void space_manager_set_layout_for_all_spaces(struct space_manager *sm, enum view
         while (bucket) {
             if (bucket->value) {
                 struct view *view = bucket->value;
-                if (!view->custom_layout) {
+                if (!view_check_flag(view, VIEW_LAYOUT)) {
                     if (space_is_user(view->sid)) {
                         view->layout = layout;
                         view_clear(view);
@@ -267,47 +280,117 @@ void space_manager_set_layout_for_all_spaces(struct space_manager *sm, enum view
     }
 }
 
-#define VIEW_SET_PROPERTY(p) \
-    sm->p = p; \
-    for (int i = 0; i < sm->view.capacity; ++i) { \
-        struct bucket *bucket = sm->view.buckets[i]; \
-        while (bucket) { \
-            if (bucket->value) { \
-                struct view *view = bucket->value; \
-                if (!view->custom_##p) view->p = p; \
-                view_update(view); \
-                view_flush(view); \
-            } \
-            bucket = bucket->next; \
-        } \
-    }
-
 void space_manager_set_window_gap_for_all_spaces(struct space_manager *sm, int window_gap)
 {
-    VIEW_SET_PROPERTY(window_gap);
+    sm->window_gap = window_gap;
+    for (int i = 0; i < sm->view.capacity; ++i) {
+        struct bucket *bucket = sm->view.buckets[i];
+        while (bucket) {
+            if (bucket->value) {
+                struct view *view = bucket->value;
+                if (!view_check_flag(view, VIEW_WINDOW_GAP)) {
+                    view->window_gap = window_gap;
+                    view_update(view);
+                    view_flush(view);
+                }
+            }
+            bucket = bucket->next;
+        }
+    }
 }
 
 void space_manager_set_top_padding_for_all_spaces(struct space_manager *sm, int top_padding)
 {
-    VIEW_SET_PROPERTY(top_padding);
+    sm->top_padding = top_padding;
+    for (int i = 0; i < sm->view.capacity; ++i) {
+        struct bucket *bucket = sm->view.buckets[i];
+        while (bucket) {
+            if (bucket->value) {
+                struct view *view = bucket->value;
+                if (!view_check_flag(view, VIEW_TOP_PADDING)) {
+                    view->top_padding = top_padding;
+                    view_update(view);
+                    view_flush(view);
+                }
+            }
+            bucket = bucket->next;
+        }
+    }
 }
 
 void space_manager_set_bottom_padding_for_all_spaces(struct space_manager *sm, int bottom_padding)
 {
-    VIEW_SET_PROPERTY(bottom_padding);
+    sm->bottom_padding = bottom_padding;
+    for (int i = 0; i < sm->view.capacity; ++i) {
+        struct bucket *bucket = sm->view.buckets[i];
+        while (bucket) {
+            if (bucket->value) {
+                struct view *view = bucket->value;
+                if (!view_check_flag(view, VIEW_BOTTOM_PADDING)) {
+                    view->bottom_padding = bottom_padding;
+                    view_update(view);
+                    view_flush(view);
+                }
+            }
+            bucket = bucket->next;
+        }
+    }
 }
 
 void space_manager_set_left_padding_for_all_spaces(struct space_manager *sm, int left_padding)
 {
-    VIEW_SET_PROPERTY(left_padding);
+    sm->left_padding = left_padding;
+    for (int i = 0; i < sm->view.capacity; ++i) {
+        struct bucket *bucket = sm->view.buckets[i];
+        while (bucket) {
+            if (bucket->value) {
+                struct view *view = bucket->value;
+                if (!view_check_flag(view, VIEW_LEFT_PADDING)) {
+                    view->left_padding = left_padding;
+                    view_update(view);
+                    view_flush(view);
+                }
+            }
+            bucket = bucket->next;
+        }
+    }
 }
 
 void space_manager_set_right_padding_for_all_spaces(struct space_manager *sm, int right_padding)
 {
-    VIEW_SET_PROPERTY(right_padding);
+    sm->right_padding = right_padding;
+    for (int i = 0; i < sm->view.capacity; ++i) {
+        struct bucket *bucket = sm->view.buckets[i];
+        while (bucket) {
+            if (bucket->value) {
+                struct view *view = bucket->value;
+                if (!view_check_flag(view, VIEW_RIGHT_PADDING)) {
+                    view->right_padding = right_padding;
+                    view_update(view);
+                    view_flush(view);
+                }
+            }
+            bucket = bucket->next;
+        }
+    }
 }
 
-#undef VIEW_SET_PROPERTY
+void space_manager_set_auto_balance_for_all_spaces(struct space_manager *sm, bool auto_balance)
+{
+    sm->auto_balance = auto_balance;
+    for (int i = 0; i < sm->view.capacity; ++i) {
+        struct bucket *bucket = sm->view.buckets[i];
+        while (bucket) {
+            if (bucket->value) {
+                struct view *view = bucket->value;
+                if (!view_check_flag(view, VIEW_AUTO_BALANCE)) {
+                    view->auto_balance = auto_balance;
+                }
+            }
+            bucket = bucket->next;
+        }
+    }
+}
 
 bool space_manager_set_padding_for_space(struct space_manager *sm, uint64_t sid, int type, int top, int bottom, int left, int right)
 {
@@ -337,7 +420,12 @@ bool space_manager_toggle_padding_for_space(struct space_manager *sm, uint64_t s
     struct view *view = space_manager_find_view(sm, sid);
     if (view->layout == VIEW_FLOAT) return false;
 
-    view->enable_padding = !view->enable_padding;
+    if (view_check_flag(view, VIEW_ENABLE_PADDING)) {
+        view_clear_flag(view, VIEW_ENABLE_PADDING);
+    } else {
+        view_set_flag(view, VIEW_ENABLE_PADDING);
+    }
+
     view_update(view);
     view_flush(view);
 
@@ -368,7 +456,7 @@ bool space_manager_mirror_space(struct space_manager *sm, uint64_t sid, enum win
     return true;
 }
 
-bool space_manager_balance_space(struct space_manager *sm, uint64_t sid, uint32_t axis_flag)
+bool space_manager_equalize_space(struct space_manager *sm, uint64_t sid, uint32_t axis_flag)
 {
     struct view *view = space_manager_find_view(sm, sid);
     if (view->layout != VIEW_BSP) return false;
@@ -380,6 +468,18 @@ bool space_manager_balance_space(struct space_manager *sm, uint64_t sid, uint32_
     return true;
 }
 
+bool space_manager_balance_space(struct space_manager *sm, uint64_t sid, uint32_t axis_flag)
+{
+    struct view *view = space_manager_find_view(sm, sid);
+    if (view->layout != VIEW_BSP) return false;
+
+    window_node_balance(view->root, axis_flag);
+    view_update(view);
+    view_flush(view);
+
+    return true;
+}
+
 struct view *space_manager_tile_window_on_space_with_insertion_point(struct space_manager *sm, struct window *window, uint64_t sid, uint32_t insertion_point)
 {
     struct view *view = space_manager_find_view(sm, sid);
@@ -392,7 +492,7 @@ struct view *space_manager_tile_window_on_space_with_insertion_point(struct spac
     if (space_is_visible(view->sid)) {
         window_node_flush(node);
     } else {
-        view->is_dirty = true;
+        view_set_flag(view, VIEW_IS_DIRTY);
     }
 
     return view;
@@ -405,20 +505,24 @@ struct view *space_manager_tile_window_on_space(struct space_manager *sm, struct
 
 void space_manager_toggle_window_split(struct space_manager *sm, struct window *window)
 {
-    struct view *view = space_manager_find_view(sm, window_space(window));
+    struct view *view = space_manager_find_view(sm, window_space(window->id));
     if (view->layout != VIEW_BSP) return;
 
     struct window_node *node = view_find_window_node(view, window->id);
     if (node && window_node_is_intermediate(node)) {
         node->parent->split = node->parent->split == SPLIT_Y ? SPLIT_X : SPLIT_Y;
 
-        if (sm->auto_balance) {
-            window_node_equalize(view->root, SPLIT_X | SPLIT_Y);
+        if (view->auto_balance) {
+            window_node_balance(view->root, SPLIT_X | SPLIT_Y);
             view_update(view);
             view_flush(view);
         } else {
             window_node_update(view, node->parent);
-            window_node_flush(node->parent);
+            if (space_is_visible(view->sid)) {
+                window_node_flush(node->parent);
+            } else {
+                view_set_flag(view, VIEW_IS_DIRTY);
+            }
         }
     }
 }
@@ -583,7 +687,7 @@ uint64_t space_manager_active_space(void)
     uint32_t did = 0;
     struct window *window = window_manager_focused_window(&g_window_manager);
 
-    if (window) did = window_display_id(window);
+    if (window) did = window_display_id(window->id);
     if (!did)   did = display_manager_active_display_id();
     if (!did)   return 0;
 
@@ -592,35 +696,15 @@ uint64_t space_manager_active_space(void)
 
 void space_manager_move_window_to_space(uint64_t sid, struct window *window)
 {
-    CFArrayRef window_list_ref = cfarray_of_cfnumbers(&window->id, sizeof(uint32_t), 1, kCFNumberSInt32Type);
-    SLSMoveWindowsToManagedSpace(g_connection, window_list_ref, sid);
-    CFRelease(window_list_ref);
-}
-
-enum space_op_error space_manager_focus_space(uint64_t sid)
-{
-    bool is_in_mc = mission_control_is_active();
-    if (is_in_mc) return SPACE_OP_ERROR_IN_MISSION_CONTROL;
-
-    uint64_t cur_sid = space_manager_active_space();
-    if (cur_sid == sid) return SPACE_OP_ERROR_SAME_SPACE;
-
-    uint32_t cur_did = space_display_id(cur_sid);
-    uint32_t new_did = space_display_id(sid);
-    bool focus_display = cur_did != new_did;
-
-    bool is_animating = display_manager_display_is_animating(new_did);
-    if (is_animating) return SPACE_OP_ERROR_DISPLAY_IS_ANIMATING;
-
-    if (scripting_addition_focus_space(sid)) {
-        if (focus_display) {
-            display_manager_focus_display(new_did, sid);
-        }
+    if (workspace_is_macos_sonoma14_5_or_newer()) {
+        SLSSpaceSetCompatID(g_connection, sid, 0x79616265);
+        SLSSetWindowListWorkspace(g_connection, &window->id, 1, 0x79616265);
+        SLSSpaceSetCompatID(g_connection, sid, 0x0);
     } else {
-        return SPACE_OP_ERROR_SCRIPTING_ADDITION;
+        CFArrayRef window_list_ref = cfarray_of_cfnumbers(&window->id, sizeof(uint32_t), 1, kCFNumberSInt32Type);
+        SLSMoveWindowsToManagedSpace(g_connection, window_list_ref, sid);
+        CFRelease(window_list_ref);
     }
-
-    return SPACE_OP_ERROR_SUCCESS;
 }
 
 static inline uint64_t space_manager_find_first_user_space_for_display(uint32_t did)
@@ -661,6 +745,66 @@ static inline bool space_manager_is_space_last_user_space(uint64_t sid)
     return result;
 }
 
+static enum space_op_error space_manager_swap_space_with_space_on_display(uint32_t a_did, uint64_t a_sid, uint32_t b_did, uint64_t b_sid)
+{
+    if (display_manager_display_is_animating(a_did)) return SPACE_OP_ERROR_DISPLAY_IS_ANIMATING;
+    if (display_manager_display_is_animating(b_did)) return SPACE_OP_ERROR_DISPLAY_IS_ANIMATING;
+
+    float window_animation_duration = g_window_manager.window_animation_duration;
+    g_window_manager.window_animation_duration = 0.0f;
+    __asm__ __volatile__ ("" ::: "memory");
+
+    int a_window_list_count = 0;
+    uint32_t *a_window_list = space_window_list(a_sid, &a_window_list_count, true);
+
+    int b_window_list_count = 0;
+    uint32_t *b_window_list = space_window_list(b_sid, &b_window_list_count, true);
+
+    struct view *a_view = table_find(&g_space_manager.view, &a_sid);
+    struct view *b_view = table_find(&g_space_manager.view, &b_sid);
+
+    table_remove(&g_space_manager.view, &a_sid);
+    table_remove(&g_space_manager.view, &b_sid);
+
+    a_view->sid = b_sid;
+    b_view->sid = a_sid;
+
+    CFStringRef tmp = a_view->uuid;
+    a_view->uuid    = b_view->uuid;
+    b_view->uuid    = tmp;
+
+    table_add(&g_space_manager.view, &a_sid, b_view);
+    table_add(&g_space_manager.view, &b_sid, a_view);
+
+    if (a_window_list_count) {
+        CFArrayRef window_list_ref = cfarray_of_cfnumbers(a_window_list, sizeof(uint32_t), a_window_list_count, kCFNumberSInt32Type);
+        SLSMoveWindowsToManagedSpace(g_connection, window_list_ref, b_sid);
+        CFRelease(window_list_ref);
+    }
+
+    if (b_window_list_count) {
+        CFArrayRef window_list_ref = cfarray_of_cfnumbers(b_window_list, sizeof(uint32_t), b_window_list_count, kCFNumberSInt32Type);
+        SLSMoveWindowsToManagedSpace(g_connection, window_list_ref, a_sid);
+        CFRelease(window_list_ref);
+    }
+
+    for (int i = 0; i < buf_len(g_space_manager.labels); ++i) {
+        struct space_label *label = &g_space_manager.labels[i];
+        if      (label->sid == a_sid) label->sid = b_sid;
+        else if (label->sid == b_sid) label->sid = a_sid;
+    }
+
+    view_update(a_view);
+    view_update(b_view);
+
+    view_flush(a_view);
+    view_flush(b_view);
+
+    __asm__ __volatile__ ("" ::: "memory");
+    g_window_manager.window_animation_duration = window_animation_duration;
+    return SPACE_OP_ERROR_SUCCESS;
+}
+
 enum space_op_error space_manager_swap_space_with_space(uint64_t acting_sid, uint64_t selector_sid)
 {
     bool is_in_mc = mission_control_is_active();
@@ -670,7 +814,7 @@ enum space_op_error space_manager_swap_space_with_space(uint64_t acting_sid, uin
     uint32_t selector_did = space_display_id(selector_sid);
 
     if (acting_sid == selector_sid) return SPACE_OP_ERROR_SAME_SPACE;
-    if (acting_did != selector_did) return SPACE_OP_ERROR_SAME_DISPLAY;
+    if (acting_did != selector_did) return space_manager_swap_space_with_space_on_display(acting_did, acting_sid, selector_did, selector_sid);
 
     bool is_animating = display_manager_display_is_animating(acting_did);
     if (is_animating) return SPACE_OP_ERROR_DISPLAY_IS_ANIMATING;
@@ -772,15 +916,71 @@ enum space_op_error space_manager_move_space_to_display(struct space_manager *sm
     uint64_t d_sid = display_space_id(did);
     if (!d_sid) return SPACE_OP_ERROR_MISSING_DST;
 
-    if (scripting_addition_move_space_after_space(sid, d_sid, 1)) {
+    bool focus_space = sid == space_manager_active_space();
+
+    if (scripting_addition_move_space_to_display(sid, d_sid,  focus_space ? space_manager_prev_space(sid) : 0, focus_space ? 1 : 0)) {
         space_manager_mark_view_invalid(sm, sid);
-        space_manager_focus_space(sid);
+        if (focus_space) {
+            space_manager_focus_space(sid);
+        }
         return SPACE_OP_ERROR_SUCCESS;
     }
 
     return SPACE_OP_ERROR_SCRIPTING_ADDITION;
 }
 
+enum space_op_error space_manager_focus_space(uint64_t sid)
+{
+    bool is_in_mc = mission_control_is_active();
+    if (is_in_mc) return SPACE_OP_ERROR_IN_MISSION_CONTROL;
+
+    uint64_t cur_sid = space_manager_active_space();
+    if (cur_sid == sid) return SPACE_OP_ERROR_SAME_SPACE;
+
+    uint32_t cur_did = space_display_id(cur_sid);
+    uint32_t new_did = space_display_id(sid);
+    bool focus_display = cur_did != new_did;
+
+    bool is_animating = display_manager_display_is_animating(new_did);
+    if (is_animating) return SPACE_OP_ERROR_DISPLAY_IS_ANIMATING;
+
+    if (scripting_addition_focus_space(sid)) {
+        if (focus_display) {
+            display_manager_focus_display(new_did, sid);
+        }
+    } else {
+        return SPACE_OP_ERROR_SCRIPTING_ADDITION;
+    }
+
+    return SPACE_OP_ERROR_SUCCESS;
+}
+
+enum space_op_error space_manager_switch_space(uint64_t sid)
+{
+    bool is_in_mc = mission_control_is_active();
+    if (is_in_mc) return SPACE_OP_ERROR_IN_MISSION_CONTROL;
+
+    uint64_t cur_sid = space_manager_active_space();
+    if (cur_sid == sid) return SPACE_OP_ERROR_SAME_SPACE;
+
+    uint32_t cur_did = space_display_id(cur_sid);
+    uint32_t did     = space_display_id(sid);
+
+    bool is_src_animating = display_manager_display_is_animating(cur_did);
+    if (is_src_animating) return SPACE_OP_ERROR_DISPLAY_IS_ANIMATING;
+
+    bool is_dst_animating = display_manager_display_is_animating(did);
+    if (is_dst_animating) return SPACE_OP_ERROR_DISPLAY_IS_ANIMATING;
+
+    if (cur_did != did) {
+        space_manager_swap_space_with_space_on_display(cur_did, cur_sid, did, sid);
+        display_manager_focus_display(cur_did, cur_sid);
+        return SPACE_OP_ERROR_SUCCESS;
+    }
+
+    return scripting_addition_focus_space(sid) ? SPACE_OP_ERROR_SUCCESS : SPACE_OP_ERROR_SCRIPTING_ADDITION;
+}
+
 enum space_op_error space_manager_destroy_space(uint64_t sid)
 {
     bool is_in_mc = mission_control_is_active();
@@ -838,7 +1038,7 @@ bool space_manager_is_window_on_active_space(struct window *window)
 bool space_manager_is_window_on_space(uint64_t sid, struct window *window)
 {
     int space_count;
-    uint64_t *space_list = window_space_list(window, &space_count);
+    uint64_t *space_list = window_space_list(window->id, &space_count);
     if (!space_list) return false;
 
     for (int i = 0; i < space_count; ++i) {
@@ -886,7 +1086,11 @@ bool space_manager_refresh_application_windows(struct space_manager *sm)
     for (int i = 0; i < refresh_count; ++i) {
         struct application *application = g_window_manager.applications_to_refresh[i];
         debug("%s: %s has windows that are not yet resolved\n", __FUNCTION__, application->name);
-        window_manager_add_existing_application_windows(sm, &g_window_manager, application, i);
+        bool result = window_manager_add_existing_application_windows(sm, &g_window_manager, application, i);
+        if (result) {
+            --refresh_count;
+            --i;
+        }
     }
 
     return window_count != g_window_manager.window.count;
@@ -908,7 +1112,7 @@ void space_manager_handle_display_add(struct space_manager *sm, uint32_t did)
             if (bucket->value) {
                 struct view *view = bucket->value;
                 view_list[list_count] = view;
-                uuid_list[list_count] = view->suuid;
+                uuid_list[list_count] = view->uuid;
                 ++list_count;
             }
             bucket = bucket->next;
@@ -931,13 +1135,13 @@ void space_manager_handle_display_add(struct space_manager *sm, uint32_t did)
                 view_list[j] = NULL;
 
                 table_remove(&sm->view, &view->sid);
-                CFRelease(view->suuid);
+                CFRelease(view->uuid);
 
                 struct space_label *label = space_manager_get_label_for_space(sm, view->sid);
                 if (label) label->sid = sid;
 
                 view->sid = sid;
-                view->suuid = CFRetain(uuid);
+                view->uuid = CFRetain(uuid);
 
                 table_add(&sm->view, &sid, view);
                 break;
