@@ -42,10 +42,10 @@ enum space_op_error
     SPACE_OP_ERROR_SCRIPTING_ADDITION   = 10,
 };
 
-bool space_manager_query_space(FILE *rsp, uint64_t sid);
-bool space_manager_query_spaces_for_window(FILE *rsp, struct window *window);
-bool space_manager_query_spaces_for_display(FILE *rsp, uint32_t did);
-bool space_manager_query_spaces_for_displays(FILE *rsp);
+bool space_manager_query_space(FILE *rsp, uint64_t sid, uint64_t flags);
+bool space_manager_query_spaces_for_window(FILE *rsp, struct window *window, uint64_t flags);
+bool space_manager_query_spaces_for_display(FILE *rsp, uint32_t did, uint64_t flags);
+bool space_manager_query_spaces_for_displays(FILE *rsp, uint64_t flags);
 struct view *space_manager_query_view(struct space_manager *sm, uint64_t sid);
 struct view *space_manager_find_view(struct space_manager *sm, uint64_t sid);
 void space_manager_refresh_view(struct space_manager *sm, uint64_t sid);
@@ -54,6 +54,7 @@ void space_manager_mark_view_dirty(struct space_manager *sm,  uint64_t sid);
 void space_manager_untile_window(struct space_manager *sm, struct view *view, struct window *window);
 struct view *space_manager_tile_window_on_space_with_insertion_point(struct space_manager *sm, struct window *window, uint64_t sid, uint32_t insertion_point);
 struct view *space_manager_tile_window_on_space(struct space_manager *sm, struct window *window, uint64_t sid);
+bool space_manager_equalize_space(struct space_manager *sm, uint64_t sid, uint32_t axis_flag);
 bool space_manager_balance_space(struct space_manager *sm, uint64_t sid, uint32_t axis_flag);
 void space_manager_toggle_window_split(struct space_manager *sm, struct window *window);
 int space_manager_mission_control_index(uint64_t sid);
@@ -79,12 +80,14 @@ void space_manager_set_top_padding_for_all_spaces(struct space_manager *sm, int
 void space_manager_set_bottom_padding_for_all_spaces(struct space_manager *sm, int bottom_padding);
 void space_manager_set_left_padding_for_all_spaces(struct space_manager *sm, int left_padding);
 void space_manager_set_right_padding_for_all_spaces(struct space_manager *sm, int right_padding);
+void space_manager_set_auto_balance_for_all_spaces(struct space_manager *sm, bool auto_balance);
 bool space_manager_set_padding_for_space(struct space_manager *sm, uint64_t sid, int type, int top, int bottom, int left, int right);
 bool space_manager_toggle_padding_for_space(struct space_manager *sm, uint64_t sid);
 bool space_manager_rotate_space(struct space_manager *sm, uint64_t sid, int degrees);
 bool space_manager_mirror_space(struct space_manager *sm, uint64_t sid, enum window_node_split axis);
 void space_manager_move_window_to_space(uint64_t sid, struct window *window);
 enum space_op_error space_manager_focus_space(uint64_t sid);
+enum space_op_error space_manager_switch_space(uint64_t sid);
 enum space_op_error space_manager_swap_space_with_space(uint64_t acting_sid, uint64_t selector_sid);
 enum space_op_error space_manager_move_space_to_space(uint64_t acting_sid, uint64_t selector_sid);
 enum space_op_error space_manager_move_space_to_display(struct space_manager *sm, uint64_t sid, uint32_t did);
