@@ -1,27 +1,48 @@
 extern int g_connection;
-extern int g_layer_normal_window_level;
-extern int g_layer_below_window_level;
 extern struct display_manager g_display_manager;
 extern struct space_manager g_space_manager;
 extern struct window_manager g_window_manager;
 
-#define INSERT_FEEDBACK_WIDTH 4
+void insert_feedback_update_notifications(void)
+{
+    int window_count = 0;
+    uint32_t window_list[1024] = {0};
+
+    for (int i = 0; i < g_window_manager.insert_feedback.capacity; ++i) {
+        struct bucket *bucket = g_window_manager.insert_feedback.buckets[i];
+        while (bucket) {
+            if (bucket->value) {
+                struct window_node *node = bucket->value;
+                window_list[window_count++] = node->window_order[0];
+            }
+
+            bucket = bucket->next;
+        }
+    }
+
+    SLSRequestNotificationsForWindows(g_connection, window_list, window_count);
+}
+
+#define INSERT_FEEDBACK_WIDTH 2
 #define INSERT_FEEDBACK_RADIUS 9
 void insert_feedback_show(struct window_node *node)
 {
     CFTypeRef frame_region;
     CGRect frame = {{node->area.x, node->area.y},{node->area.w, node->area.h}};
     CGSNewRegionWithRect(&frame, &frame_region);
+    frame.origin.x = 0; frame.origin.y = 0;
 
     if (!node->feedback_window.id) {
-        uint64_t tag = 1ULL << 46;
-        SLSNewWindow(g_connection, 2, 0, 0, frame_region, &node->feedback_window.id);
-        SLSSetWindowTags(g_connection, node->feedback_window.id, &tag, 64);
+        uint64_t tags = (1ULL << 1) | (1ULL << 9);
+        CFTypeRef empty_region = CGRegionCreateEmptyRegion();
+        SLSNewWindowWithOpaqueShapeAndContext(g_connection, 2, frame_region, empty_region, 13, &tags, 0, 0, 64, &node->feedback_window.id, NULL);
+        CFRelease(empty_region);
+
         sls_window_disable_shadow(node->feedback_window.id);
-        SLSSetWindowResolution(g_connection, node->feedback_window.id, 2.0f);
+        SLSSetWindowResolution(g_connection, node->feedback_window.id, 1.0f);
         SLSSetWindowOpacity(g_connection, node->feedback_window.id, 0);
-        SLSSetWindowLevel(g_connection, node->feedback_window.id, g_layer_normal_window_level);
-        SLSSetWindowSubLevel(g_connection, node->feedback_window.id, g_layer_below_window_level);
+        SLSSetWindowLevel(g_connection, node->feedback_window.id, window_level(node->window_order[0]));
+        SLSSetWindowSubLevel(g_connection, node->feedback_window.id, window_sub_level(node->window_order[0]));
         node->feedback_window.context = SLWindowContextCreate(g_connection, node->feedback_window.id, 0);
         CGContextSetLineWidth(node->feedback_window.context, INSERT_FEEDBACK_WIDTH);
         CGContextSetRGBFillColor(node->feedback_window.context,
@@ -34,10 +55,15 @@ void insert_feedback_show(struct window_node *node)
                                    g_window_manager.insert_feedback_color.g,
                                    g_window_manager.insert_feedback_color.b,
                                    g_window_manager.insert_feedback_color.a);
-        buf_push(g_window_manager.insert_feedback_windows, node->feedback_window.id);
+        SLSDisableUpdate(g_connection);
+        CGContextClearRect(node->feedback_window.context, frame);
+        CGContextFlush(node->feedback_window.context);
+        SLSReenableUpdate(g_connection);
+        SLSOrderWindow(g_connection, node->feedback_window.id, 1, node->window_order[0]);
+        table_add(&g_window_manager.insert_feedback, &node->window_order[0], node);
+        insert_feedback_update_notifications();
     }
 
-    frame.origin.x = 0; frame.origin.y = 0;
     CGFloat clip_x, clip_y, clip_w, clip_h;
     CGFloat midx = CGRectGetMidX(frame);
     CGFloat midy = CGRectGetMidY(frame);
@@ -81,7 +107,6 @@ void insert_feedback_show(struct window_node *node)
     CGPathRef path = CGPathCreateWithRoundedRect(rect, cgrect_clamp_x_radius(rect, INSERT_FEEDBACK_RADIUS), cgrect_clamp_y_radius(rect, INSERT_FEEDBACK_RADIUS), NULL);
 
     SLSDisableUpdate(g_connection);
-    SLSOrderWindow(g_connection, node->feedback_window.id, 0, node->window_order[0]);
     SLSSetWindowShape(g_connection, node->feedback_window.id, 0.0f, 0.0f, frame_region);
     CGContextClearRect(node->feedback_window.context, frame);
     CGContextClipToRect(node->feedback_window.context, clip);
@@ -90,7 +115,6 @@ void insert_feedback_show(struct window_node *node)
     CGContextStrokePath(node->feedback_window.context);
     CGContextResetClip(node->feedback_window.context);
     CGContextFlush(node->feedback_window.context);
-    SLSOrderWindow(g_connection, node->feedback_window.id, 1, node->window_order[0]);
     SLSReenableUpdate(g_connection);
     CGPathRelease(path);
     CFRelease(frame_region);
@@ -99,13 +123,10 @@ void insert_feedback_show(struct window_node *node)
 void insert_feedback_destroy(struct window_node *node)
 {
     if (node->feedback_window.id) {
-        for (int i = 0; i < buf_len(g_window_manager.insert_feedback_windows); ++i) {
-            if (g_window_manager.insert_feedback_windows[i] == node->feedback_window.id) {
-                buf_del(g_window_manager.insert_feedback_windows, i);
-                break;
-            }
-        }
+        table_remove(&g_window_manager.insert_feedback, &node->window_order[0]);
+        insert_feedback_update_notifications();
 
+        SLSOrderWindow(g_connection, node->feedback_window.id, 0, 0);
         CGContextRelease(node->feedback_window.context);
         SLSReleaseWindow(g_connection, node->feedback_window.id);
         memset(&node->feedback_window, 0, sizeof(struct feedback_window));
@@ -117,6 +138,11 @@ static inline struct area area_from_cgrect(CGRect rect)
     return (struct area) { rect.origin.x, rect.origin.y, rect.size.width, rect.size.height };
 }
 
+static inline CGPoint area_max_point(struct area area)
+{
+    return (CGPoint) { area.x + area.w - 1, area.y + area.h - 1 };
+}
+
 static inline enum window_node_child window_node_get_child(struct window_node *node)
 {
     return node->child != CHILD_NONE ? node->child : g_space_manager.window_placement;
@@ -136,7 +162,7 @@ static inline float window_node_get_ratio(struct window_node *node)
 
 static inline int window_node_get_gap(struct view *view)
 {
-    return view->enable_gap ? view->window_gap : 0;
+    return view_check_flag(view, VIEW_ENABLE_GAP) ? view->window_gap : 0;
 }
 
 static void area_make_pair(enum window_node_split split, int gap, float ratio, struct area *parent_area, struct area *left_area, struct area *right_area)
@@ -195,23 +221,37 @@ static inline bool window_node_is_right_child(struct window_node *node)
     return node->parent && node->parent->right == node;
 }
 
-static inline struct equalize_node equalize_node_add(struct equalize_node a, struct equalize_node b)
+static void window_node_equalize(struct window_node *node, uint32_t axis_flag)
 {
-    return (struct equalize_node) { a.y_count + b.y_count, a.x_count + b.x_count, };
+    if (node->left)  window_node_equalize(node->left, axis_flag);
+    if (node->right) window_node_equalize(node->right, axis_flag);
+
+    if ((axis_flag & SPLIT_Y) && node->split == SPLIT_Y) {
+        node->ratio = g_space_manager.split_ratio;
+    }
+
+    if ((axis_flag & SPLIT_X) && node->split == SPLIT_X) {
+        node->ratio = g_space_manager.split_ratio;
+    }
 }
 
-static struct equalize_node window_node_equalize(struct window_node *node, uint32_t axis_flag)
+static inline struct balance_node balance_node_add(struct balance_node a, struct balance_node b)
+{
+    return (struct balance_node) { a.y_count + b.y_count, a.x_count + b.x_count, };
+}
+
+static struct balance_node window_node_balance(struct window_node *node, uint32_t axis_flag)
 {
     if (window_node_is_leaf(node)) {
-        return (struct equalize_node) {
+        return (struct balance_node) {
             node->parent ? node->parent->split == SPLIT_Y : 0,
             node->parent ? node->parent->split == SPLIT_X : 0
         };
     }
 
-    struct equalize_node left_leafs  = window_node_equalize(node->left, axis_flag);
-    struct equalize_node right_leafs = window_node_equalize(node->right, axis_flag);
-    struct equalize_node total_leafs = equalize_node_add(left_leafs, right_leafs);
+    struct balance_node left_leafs  = window_node_balance(node->left, axis_flag);
+    struct balance_node right_leafs = window_node_balance(node->right, axis_flag);
+    struct balance_node total_leafs = balance_node_add(left_leafs, right_leafs);
 
     if (axis_flag & SPLIT_Y) {
         if (node->split == SPLIT_Y) {
@@ -284,13 +324,10 @@ static void window_node_split(struct view *view, struct window_node *node, struc
 
 void window_node_update(struct view *view, struct window_node *node)
 {
-    if (window_node_is_intermediate(node)) {
-        area_make_pair_for_node(view, node->parent);
-    }
-
     if (window_node_is_leaf(node)) {
         if (node->insert_dir) insert_feedback_show(node);
     } else {
+        area_make_pair_for_node(view, node);
         window_node_update(view, node->left);
         window_node_update(view, node->right);
     }
@@ -321,7 +358,7 @@ static void window_node_clear_zoom(struct window_node *node)
 
 void window_node_capture_windows(struct window_node *node, struct window_capture **window_list)
 {
-    if (window_node_is_occupied(node)) {
+    if (window_node_is_leaf(node)) {
         for (int i = 0; i < node->window_count; ++i) {
             struct window *window = window_manager_find_window(&g_window_manager, node->window_list[i]);
             if (window) {
@@ -329,9 +366,7 @@ void window_node_capture_windows(struct window_node *node, struct window_capture
                 ts_buf_push(*window_list, ((struct window_capture) { .window = window, .x = area.x, .y = area.y, .w = area.w, .h = area.h }));
             }
         }
-    }
-
-    if (!window_node_is_leaf(node)) {
+    } else {
         window_node_capture_windows(node->left, window_list);
         window_node_capture_windows(node->right, window_list);
     }
@@ -558,14 +593,13 @@ struct window_node *view_find_window_node_in_direction(struct view *view, struct
     int best_distance = INT_MAX;
     int best_rank = INT_MAX;
     struct window_node *best_node = NULL;
-
-    CGPoint source_area_max = { source->area.x + source->area.w - 1, source->area.y + source->area.h - 1};
+    CGPoint source_area_max = area_max_point(source->area);
 
     struct window_node *target = window_node_find_first_leaf(view->root);
     while (target) {
         if (source == target) goto next;
 
-        CGPoint target_area_max = { target->area.x + target->area.w - 1, target->area.y + target->area.h - 1 };
+        CGPoint target_area_max = area_max_point(target->area);
         if (area_is_in_direction(&source->area, source_area_max, &target->area, target_area_max, direction)) {
             int distance = area_distance_in_direction(&source->area, source_area_max, &target->area, target_area_max, direction);
             int rank = window_manager_find_rank_of_window_in_list(target->window_order[0], window_list, window_count);
@@ -687,17 +721,15 @@ struct window_node *view_remove_window_node(struct view *view, struct window *wi
             window_node_clear_zoom(parent);
         }
 
-        window_node_update(view, parent->left);
-        window_node_update(view, parent->right);
+        window_node_update(view, parent);
     }
 
     insert_feedback_destroy(node);
-
     free(child);
     free(node);
 
-    if (g_space_manager.auto_balance) {
-        window_node_equalize(view->root, SPLIT_X | SPLIT_Y);
+    if (view->auto_balance) {
+        window_node_balance(view->root, SPLIT_X | SPLIT_Y);
         view_update(view);
         return view->root;
     }
@@ -752,8 +784,8 @@ struct window_node *view_add_window_node_with_insertion_point(struct view *view,
 
         window_node_split(view, leaf, window);
 
-        if (g_space_manager.auto_balance) {
-            window_node_equalize(view->root, SPLIT_X | SPLIT_Y);
+        if (view->auto_balance) {
+            window_node_balance(view->root, SPLIT_X | SPLIT_Y);
             view_update(view);
             return view->root;
         }
@@ -798,74 +830,130 @@ uint32_t *view_find_window_list(struct view *view, int *window_count)
 
 bool view_is_invalid(struct view *view)
 {
-    return !view->is_valid;
+    return !view_check_flag(view, VIEW_IS_VALID);
 }
 
 bool view_is_dirty(struct view *view)
 {
-    return view->is_dirty;
+    return view_check_flag(view, VIEW_IS_DIRTY);
 }
 
 void view_flush(struct view *view)
 {
-    window_node_flush(view->root);
-    view->is_dirty = false;
+    if (space_is_visible(view->sid)) {
+        window_node_flush(view->root);
+        view_clear_flag(view, VIEW_IS_DIRTY);
+    } else {
+        view_set_flag(view, VIEW_IS_DIRTY);
+    }
 }
 
-void view_serialize(FILE *rsp, struct view *view)
+void view_serialize(FILE *rsp, struct view *view, uint64_t flags)
 {
-    char *uuid = ts_cfstring_copy(view->suuid);
-    int buffer_size = MAXLEN;
-    size_t bytes_written = 0;
-    char buffer[MAXLEN] = {};
-    char *cursor = buffer;
+    TIME_FUNCTION;
 
-    int window_count = 0;
-    uint32_t *window_list = space_window_list(view->sid, &window_count, true);
+    if (flags == 0x0) flags |= ~flags;
 
-    for (int i = 0; i < window_count; ++i) {
-        if (i < window_count - 1) {
-            bytes_written = snprintf(cursor, buffer_size, "%d, ", window_list[i]);
-        } else {
-            bytes_written = snprintf(cursor, buffer_size, "%d", window_list[i]);
-        }
+    bool did_output = false;
+    fprintf(rsp, "{\n");
 
-        cursor += bytes_written;
-        buffer_size -= bytes_written;
-        if (buffer_size <= 0) break;
+    if (flags & SPACE_PROPERTY_ID) {
+        fprintf(rsp, "\t\"id\":%lld", view->sid);
+        did_output = true;
     }
 
-    struct space_label *space_label = space_manager_get_label_for_space(&g_space_manager, view->sid);
-    struct window_node *first_leaf = window_node_find_first_leaf(view->root);
-    struct window_node *last_leaf = window_node_find_last_leaf(view->root);
+    if (flags & SPACE_PROPERTY_UUID) {
+        if (did_output) fprintf(rsp, ",\n");
 
-    fprintf(rsp,
-            "{\n"
-            "\t\"id\":%lld,\n"
-            "\t\"uuid\":\"%s\",\n"
-            "\t\"index\":%d,\n"
-            "\t\"label\":\"%s\",\n"
-            "\t\"type\":\"%s\",\n"
-            "\t\"display\":%d,\n"
-            "\t\"windows\":[%s],\n"
-            "\t\"first-window\":%d,\n"
-            "\t\"last-window\":%d,\n"
-            "\t\"has-focus\":%s,\n"
-            "\t\"is-visible\":%s,\n"
-            "\t\"is-native-fullscreen\":%s\n"
-            "}",
-            view->sid,
-            uuid ? uuid : "<unknown>",
-            space_manager_mission_control_index(view->sid),
-            space_label ? space_label->label : "",
-            view_type_str[view->layout],
-            display_arrangement(space_display_id(view->sid)),
-            buffer,
-            first_leaf ? first_leaf->window_order[0] : 0,
-            last_leaf ? last_leaf->window_order[0] : 0,
-            json_bool(view->sid == g_space_manager.current_space_id),
-            json_bool(space_is_visible(view->sid)),
-            json_bool(space_is_fullscreen(view->sid)));
+        char *uuid = ts_cfstring_copy(view->uuid);
+        fprintf(rsp, "\t\"uuid\":\"%s\"", uuid ? uuid : "<unknown>");
+        did_output = true;
+    }
+
+    if (flags & SPACE_PROPERTY_INDEX) {
+        if (did_output) fprintf(rsp, ",\n");
+
+        fprintf(rsp, "\t\"index\":%d", space_manager_mission_control_index(view->sid));
+        did_output = true;
+    }
+
+    if (flags & SPACE_PROPERTY_LABEL) {
+        if (did_output) fprintf(rsp, ",\n");
+
+        struct space_label *space_label = space_manager_get_label_for_space(&g_space_manager, view->sid);
+        fprintf(rsp, "\t\"label\":\"%s\"", space_label ? space_label->label : "");
+        did_output = true;
+    }
+
+    if (flags & SPACE_PROPERTY_TYPE) {
+        if (did_output) fprintf(rsp, ",\n");
+
+        fprintf(rsp, "\t\"type\":\"%s\"", view_type_str[view->layout]);
+        did_output = true;
+    }
+
+    if (flags & SPACE_PROPERTY_DISPLAY) {
+        if (did_output) fprintf(rsp, ",\n");
+
+        fprintf(rsp, "\t\"display\":%d", display_manager_display_id_arrangement(space_display_id(view->sid)));
+        did_output = true;
+    }
+
+    if (flags & SPACE_PROPERTY_WINDOWS) {
+        if (did_output) fprintf(rsp, ",\n");
+
+        int window_count = 0;
+        uint32_t *window_list = space_window_list(view->sid, &window_count, true);
+
+        fprintf(rsp, "\t\"windows\":[");
+        for (int i = 0; i < window_count; ++i) {
+            if (i < window_count - 1) {
+                fprintf(rsp, "%d, ", window_list[i]);
+            } else {
+                fprintf(rsp, "%d", window_list[i]);
+            }
+        }
+        fprintf(rsp, "]");
+        did_output = true;
+    }
+
+    if (flags & SPACE_PROPERTY_FIRST_WINDOW) {
+        if (did_output) fprintf(rsp, ",\n");
+
+        struct window_node *first_leaf = window_node_find_first_leaf(view->root);
+        fprintf(rsp, "\t\"first-window\":%d", first_leaf ? first_leaf->window_order[0] : 0);
+        did_output = true;
+    }
+
+    if (flags & SPACE_PROPERTY_LAST_WINDOW) {
+        if (did_output) fprintf(rsp, ",\n");
+
+        struct window_node *last_leaf = window_node_find_last_leaf(view->root);
+        fprintf(rsp, "\t\"last-window\":%d", last_leaf ? last_leaf->window_order[0] : 0);
+        did_output = true;
+    }
+
+    if (flags & SPACE_PROPERTY_HAS_FOCUS) {
+        if (did_output) fprintf(rsp, ",\n");
+
+        fprintf(rsp, "\t\"has-focus\":%s", json_bool(view->sid == g_space_manager.current_space_id));
+        did_output = true;
+    }
+
+    if (flags & SPACE_PROPERTY_IS_VISIBLE) {
+        if (did_output) fprintf(rsp, ",\n");
+
+        fprintf(rsp, "\t\"is-visible\":%s", json_bool(space_is_visible(view->sid)));
+        did_output = true;
+    }
+
+    if (flags & SPACE_PROPERTY_IS_FULLSCREEN) {
+        if (did_output) fprintf(rsp, ",\n");
+
+        fprintf(rsp, "\t\"is-native-fullscreen\":%s", json_bool(space_is_fullscreen(view->sid)));
+    }
+
+    fprintf(rsp, "\n}");
 }
 
 void view_update(struct view *view)
@@ -874,7 +962,7 @@ void view_update(struct view *view)
     CGRect frame = display_bounds_constrained(did);
     view->root->area = area_from_cgrect(frame);
 
-    if (view->enable_padding) {
+    if (view_check_flag(view, VIEW_ENABLE_PADDING)) {
         view->root->area.x += view->left_padding;
         view->root->area.w -= (view->left_padding + view->right_padding);
         view->root->area.y += view->top_padding;
@@ -882,8 +970,8 @@ void view_update(struct view *view)
     }
 
     window_node_update(view, view->root);
-    view->is_valid = true;
-    view->is_dirty = true;
+    view_set_flag(view, VIEW_IS_VALID);
+    view_set_flag(view, VIEW_IS_DIRTY);
 }
 
 struct view *view_create(uint64_t sid)
@@ -894,18 +982,20 @@ struct view *view_create(uint64_t sid)
     view->root = malloc(sizeof(struct window_node));
     memset(view->root, 0, sizeof(struct window_node));
 
-    view->enable_padding = true;
-    view->enable_gap = true;
     view->sid = sid;
-    view->suuid = SLSSpaceCopyName(g_connection, sid);
+    view->uuid = SLSSpaceCopyName(g_connection, sid);
+
+    view_set_flag(view, VIEW_ENABLE_PADDING);
+    view_set_flag(view, VIEW_ENABLE_GAP);
 
     if (space_is_user(view->sid)) {
-        if (!view->custom_layout)         view->layout         = g_space_manager.layout;
-        if (!view->custom_top_padding)    view->top_padding    = g_space_manager.top_padding;
-        if (!view->custom_bottom_padding) view->bottom_padding = g_space_manager.bottom_padding;
-        if (!view->custom_left_padding)   view->left_padding   = g_space_manager.left_padding;
-        if (!view->custom_right_padding)  view->right_padding  = g_space_manager.right_padding;
-        if (!view->custom_window_gap)     view->window_gap     = g_space_manager.window_gap;
+        if (!view_check_flag(view, VIEW_LAYOUT))         view->layout         = g_space_manager.layout;
+        if (!view_check_flag(view, VIEW_TOP_PADDING))    view->top_padding    = g_space_manager.top_padding;
+        if (!view_check_flag(view, VIEW_BOTTOM_PADDING)) view->bottom_padding = g_space_manager.bottom_padding;
+        if (!view_check_flag(view, VIEW_LEFT_PADDING))   view->left_padding   = g_space_manager.left_padding;
+        if (!view_check_flag(view, VIEW_RIGHT_PADDING))  view->right_padding  = g_space_manager.right_padding;
+        if (!view_check_flag(view, VIEW_WINDOW_GAP))     view->window_gap     = g_space_manager.window_gap;
+        if (!view_check_flag(view, VIEW_AUTO_BALANCE))   view->auto_balance   = g_space_manager.auto_balance;
         view_update(view);
     } else {
         view->layout = VIEW_FLOAT;
