@@ -65,6 +65,12 @@ static const char *window_origin_mode_str[] =
     "cursor"
 };
 
+struct scratchpad
+{
+    char *label;
+    struct window *window;
+};
+
 struct window_manager
 {
     AXUIElementRef system_element;
@@ -74,6 +80,7 @@ struct window_manager
     struct table window_lost_focused_event;
     struct table application_lost_front_switched_event;
     struct table window_animations_table;
+    struct table insert_feedback;
     pthread_mutex_t window_animations_lock;
     struct rule *rules;
     struct application **applications_to_refresh;
@@ -85,22 +92,25 @@ struct window_manager
     enum purify_mode purify_mode;
     enum window_origin_mode window_origin_mode;
     bool enable_window_opacity;
+    float menubar_opacity;
     float active_window_opacity;
     float normal_window_opacity;
     float window_opacity_duration;
     float window_animation_duration;
-    uint32_t *insert_feedback_windows;
+    int window_animation_easing;
     struct rgba_color insert_feedback_color;
+    struct scratchpad *scratchpad_window;
 };
 
 void window_manager_query_window_rules(FILE *rsp);
-void window_manager_query_windows_for_spaces(FILE *rsp, uint64_t *space_list, int space_count);
-void window_manager_query_windows_for_display(FILE *rsp, uint32_t did);
-void window_manager_query_windows_for_displays(FILE *rsp);
-void window_manager_apply_manage_rule_to_window(struct space_manager *sm, struct window_manager *wm, struct window *window, struct rule *rule, char *window_title, char *window_role, char *window_subrole);
-void window_manager_apply_rule_to_window(struct space_manager *sm, struct window_manager *wm, struct window *window, struct rule *rule, char *window_title, char *window_role, char *window_subrole);
-void window_manager_apply_manage_rules_to_window(struct space_manager *sm, struct window_manager *wm, struct window *window, char *window_title, char *window_role, char *window_subrole);
-void window_manager_apply_rules_to_window(struct space_manager *sm, struct window_manager *wm, struct window *window, char *window_title, char *window_role, char *window_subrole);
+void window_manager_query_windows_for_spaces(FILE *rsp, uint64_t *space_list, int space_count, uint64_t flags);
+void window_manager_query_windows_for_display(FILE *rsp, uint32_t did, uint64_t flags);
+void window_manager_query_windows_for_displays(FILE *rsp, uint64_t flags);
+bool window_manager_rule_matches_window(struct rule *rule, struct window *window, char *window_title, char *window_role, char *window_subrole);
+void window_manager_apply_manage_rule_effects_to_window(struct space_manager *sm, struct window_manager *wm, struct window *window, struct rule_effects *effects, char *window_title, char *window_role, char *window_subrole);
+void window_manager_apply_rule_effects_to_window(struct space_manager *sm, struct window_manager *wm, struct window *window, struct rule_effects *effects, char *window_title, char *window_role, char *window_subrole);
+void window_manager_apply_manage_rules_to_window(struct space_manager *sm, struct window_manager *wm, struct window *window, char *window_title, char *window_role, char *window_subrole, bool one_shot_rules);
+void window_manager_apply_rules_to_window(struct space_manager *sm, struct window_manager *wm, struct window *window, char *window_title, char *window_role, char *window_subrole, bool one_shot_rules);
 void window_manager_center_mouse(struct window_manager *wm, struct window *window);
 bool window_manager_is_window_eligible(struct window *window);
 bool window_manager_should_manage_window(struct window *window);
@@ -159,6 +169,7 @@ enum window_op_error window_manager_move_window_relative(struct window_manager *
 void window_manager_resize_window_relative_internal(struct window *window, CGRect frame, int direction, float dx, float dy, bool animate);
 enum window_op_error window_manager_resize_window_relative(struct window_manager *wm, struct window *window, int direction, float dx, float dy, bool animate);
 void window_manager_set_purify_mode(struct window_manager *wm, enum purify_mode mode);
+void window_manager_set_menubar_opacity(struct window_manager *wm, float opacity);
 void window_manager_set_active_window_opacity(struct window_manager *wm, float opacity);
 void window_manager_set_normal_window_opacity(struct window_manager *wm, float opacity);
 void window_manager_set_window_opacity_enabled(struct window_manager *wm, bool enabled);
@@ -173,9 +184,9 @@ enum window_op_error window_manager_minimize_window(struct window *window);
 enum window_op_error window_manager_deminimize_window(struct window *window);
 bool window_manager_close_window(struct window *window);
 void window_manager_send_window_to_space(struct space_manager *sm, struct window_manager *wm, struct window *window, uint64_t sid, bool moved_by_rule);
-struct window *window_manager_create_and_add_window(struct space_manager *sm, struct window_manager *wm, struct application *application, AXUIElementRef window_ref, uint32_t window_id);
+struct window *window_manager_create_and_add_window(struct space_manager *sm, struct window_manager *wm, struct application *application, AXUIElementRef window_ref, uint32_t window_id, bool one_shot_rules);
 struct window **window_manager_add_application_windows(struct space_manager *sm, struct window_manager *wm, struct application *application, int *count);
-void window_manager_add_existing_application_windows(struct space_manager *sm, struct window_manager *wm, struct application *application, int refresh_index);
+bool window_manager_add_existing_application_windows(struct space_manager *sm, struct window_manager *wm, struct application *application, int refresh_index);
 enum window_op_error window_manager_apply_grid(struct space_manager *sm, struct window_manager *wm, struct window *window, unsigned r, unsigned c, unsigned x, unsigned y, unsigned w, unsigned h);
 void window_manager_purify_window(struct window_manager *wm, struct window *window);
 void window_manager_make_window_floating(struct space_manager *sm, struct window_manager *wm, struct window *window, bool should_float, bool force);
@@ -188,6 +199,11 @@ void window_manager_toggle_window_fullscreen(struct space_manager *sm, struct wi
 void window_manager_toggle_window_native_fullscreen(struct space_manager *sm, struct window_manager *wm, struct window *window);
 void window_manager_toggle_window_expose(struct window_manager *wm, struct window *window);
 void window_manager_toggle_window_pip(struct space_manager *sm, struct window_manager *wm, struct window *window);
+bool window_manager_toggle_scratchpad_window_by_label(struct window_manager *wm, char *label);
+bool window_manager_toggle_scratchpad_window(struct window_manager *wm, struct window *window, int forced_mode);
+bool window_manager_set_scratchpad_for_window(struct window_manager *wm, struct window *window, char *label);
+bool window_manager_remove_scratchpad_for_window(struct window_manager *wm, struct window *window, bool unfloat);
+void window_manager_scratchpad_recover_windows(void);
 void window_manager_wait_for_native_fullscreen_transition(struct window *window);
 void window_manager_validate_and_check_for_windows_on_space(struct space_manager *sm, struct window_manager *wm, uint64_t sid);
 void window_manager_correct_for_mission_control_changes(struct space_manager *sm, struct window_manager *wm);
