@@ -14,6 +14,17 @@ bool workspace_event_handler_begin(void **context)
     return true;
 }
 
+bool workspace_is_macos_sonoma14_5_or_newer(void)
+{
+    NSOperatingSystemVersion os_version = [[NSProcessInfo processInfo] operatingSystemVersion];
+
+    if (os_version.majorVersion > 14) return true;
+
+    if (os_version.majorVersion == 14 && os_version.minorVersion >= 5) return true;
+
+    return false;
+}
+
 void *workspace_application_create_running_ns_application(struct process *process)
 {
     return [[NSRunningApplication runningApplicationWithProcessIdentifier:process->pid] retain];
@@ -79,8 +90,10 @@ bool workspace_application_is_observable(struct process *process)
 {
     NSRunningApplication *application = __atomic_load_n(&process->ns_application, __ATOMIC_RELAXED);
     if (application) {
-        return [application activationPolicy] != NSApplicationActivationPolicyProhibited;
+        process->policy = [application activationPolicy];
+        return process->policy == NSApplicationActivationPolicyRegular;
     } else {
+        process->policy = NSApplicationActivationPolicyProhibited;
         return false;
     }
 }
@@ -152,16 +165,16 @@ extern struct event_loop g_event_loop;
                 name:NSWorkspaceDidWakeNotification
                 object:nil];
 
-       [[NSNotificationCenter defaultCenter] addObserver:self
-                selector:@selector(didRestartDock:)
-                name:@"NSApplicationDockDidRestartNotification"
-                object:nil];
-
        [[NSDistributedNotificationCenter defaultCenter] addObserver:self
                 selector:@selector(didChangeMenuBarHiding:)
                 name:@"AppleInterfaceMenuBarHidingChangedNotification"
                 object:nil];
 
+       [[NSNotificationCenter defaultCenter] addObserver:self
+                selector:@selector(didRestartDock:)
+                name:@"NSApplicationDockDidRestartNotification"
+                object:nil];
+
        [[NSDistributedNotificationCenter defaultCenter] addObserver:self
                 selector:@selector(didChangeDockPref:)
                 name:@"com.apple.dock.prefchanged"
@@ -182,14 +195,11 @@ extern struct event_loop g_event_loop;
 - (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
 {
     if ([keyPath isEqualToString:@"activationPolicy"]) {
+        struct process *process = context;
+        assert(!process->terminated);
+
         id result = [change objectForKey:NSKeyValueChangeNewKey];
-        if ([result intValue] != NSApplicationActivationPolicyProhibited) {
-            struct process *process = context;
-            assert(!process->terminated);
-
-            debug("%s: activation policy changed for %s (%d)\n", __FUNCTION__, process->name, process->pid);
-            event_loop_post(&g_event_loop, APPLICATION_LAUNCHED, process, 0);
-
+        if ([result intValue] != process->policy) {
             //
             // :WorstApiEverMade
             //
@@ -202,18 +212,19 @@ extern struct event_loop g_event_loop;
             @try {
                 [object removeObserver:self forKeyPath:@"activationPolicy" context:process];
             } @catch (NSException * __unused exception) {}
+
+            process->policy = [result intValue];
+            debug("%s: activation policy changed for %s (%d)\n", __FUNCTION__, process->name, process->pid);
+            event_loop_post(&g_event_loop, APPLICATION_LAUNCHED, process, 0);
         }
     }
 
     if ([keyPath isEqualToString:@"finishedLaunching"]) {
+        struct process *process = context;
+        assert(!process->terminated);
+
         id result = [change objectForKey:NSKeyValueChangeNewKey];
         if ([result intValue] == 1) {
-            struct process *process = context;
-            assert(!process->terminated);
-
-            debug("%s: %s (%d) finished launching\n", __FUNCTION__, process->name, process->pid);
-            event_loop_post(&g_event_loop, APPLICATION_LAUNCHED, process, 0);
-
             //
             // :WorstApiEverMade
             //
@@ -226,6 +237,9 @@ extern struct event_loop g_event_loop;
             @try {
                 [object removeObserver:self forKeyPath:@"finishedLaunching" context:process];
             } @catch (NSException * __unused exception) {}
+
+            debug("%s: %s (%d) finished launching\n", __FUNCTION__, process->name, process->pid);
+            event_loop_post(&g_event_loop, APPLICATION_LAUNCHED, process, 0);
         }
     }
 }
@@ -235,16 +249,16 @@ extern struct event_loop g_event_loop;
     event_loop_post(&g_event_loop, SYSTEM_WOKE, NULL, 0);
 }
 
-- (void)didRestartDock:(NSNotification *)notification
-{
-    event_loop_post(&g_event_loop, DOCK_DID_RESTART, NULL, 0);
-}
-
 - (void)didChangeMenuBarHiding:(NSNotification *)notification
 {
     event_loop_post(&g_event_loop, MENU_BAR_HIDDEN_CHANGED, NULL, 0);
 }
 
+- (void)didRestartDock:(NSNotification *)notification
+{
+    event_loop_post(&g_event_loop, DOCK_DID_RESTART, NULL, 0);
+}
+
 - (void)didChangeDockPref:(NSNotification *)notification
 {
     event_loop_post(&g_event_loop, DOCK_DID_CHANGE_PREF, NULL, 0);
