@@ -20,32 +20,36 @@
 #define HELP_OPT_LONG           "--help"
 #define HELP_OPT_SHRT           "-h"
 
-#define MAJOR  6
-#define MINOR  0
-#define PATCH 10
+#define MAJOR  7
+#define MINOR  1
+#define PATCH  1
 
-struct event_loop g_event_loop;
-void *g_workspace_context;
+struct signal *g_signal_event[SIGNAL_TYPE_COUNT];
 struct process_manager g_process_manager;
 struct display_manager g_display_manager;
-struct space_manager g_space_manager;
 struct window_manager g_window_manager;
+struct space_manager g_space_manager;
+struct memory_pool g_signal_storage;
 struct mouse_state g_mouse_state;
+struct event_loop g_event_loop;
+void *g_workspace_context;
+
+enum mission_control_mode g_mission_control_mode;
 double g_cv_host_clock_frequency;
 int g_layer_normal_window_level;
 int g_layer_below_window_level;
 int g_layer_above_window_level;
-int g_connection;
-pid_t g_pid;
+uint8_t *g_event_bytes;
 
-struct signal *g_signal_event[SIGNAL_TYPE_COUNT];
-struct memory_pool g_signal_storage;
-enum mission_control_mode g_mission_control_mode;
 char g_sa_socket_file[MAXLEN];
 char g_socket_file[MAXLEN];
 char g_config_file[4096];
 char g_lock_file[MAXLEN];
+
+mach_port_t g_bs_port;
+int g_connection;
 bool g_verbose;
+pid_t g_pid;
 
 static int client_send_message(int argc, char **argv)
 {
@@ -119,50 +123,6 @@ static int client_send_message(int argc, char **argv)
     return result;
 }
 
-static bool get_config_file(char *restrict filename, char *restrict buffer, int buffer_size)
-{
-    char *xdg_home = getenv("XDG_CONFIG_HOME");
-    if (xdg_home && *xdg_home) {
-        snprintf(buffer, buffer_size, "%s/yabai/%s", xdg_home, filename);
-        if (file_exists(buffer)) return true;
-    }
-
-    char *home = getenv("HOME");
-    if (!home) return false;
-
-    snprintf(buffer, buffer_size, "%s/.config/yabai/%s", home, filename);
-    if (file_exists(buffer)) return true;
-
-    snprintf(buffer, buffer_size, "%s/.%s", home, filename);
-    return file_exists(buffer);
-}
-
-static void exec_config_file(void)
-{
-    if (!*g_config_file && !get_config_file("yabairc", g_config_file, sizeof(g_config_file))) {
-        notify("configuration", "could not locate config file..");
-        return;
-    }
-
-    if (!file_exists(g_config_file)) {
-        notify("configuration", "file '%s' does not exist..", g_config_file);
-        return;
-    }
-
-    if (!ensure_executable_permission(g_config_file)) {
-        notify("configuration", "could not set the executable permission bit for '%s'", g_config_file);
-        return;
-    }
-
-    int pid = fork();
-    if (pid == 0) {
-        char *exec[] = { "/usr/bin/env", "sh", "-c", g_config_file, NULL};
-        exit(execvp(exec[0], exec));
-    } else if (pid == -1) {
-        notify("configuration", "failed to execute file '%s'", g_config_file);
-    }
-}
-
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wdeprecated-declarations"
 static inline bool configure_settings_and_acquire_lock(void)
@@ -176,7 +136,9 @@ static inline bool configure_settings_and_acquire_lock(void)
     snprintf(g_socket_file, sizeof(g_socket_file), SOCKET_PATH_FMT, user);
     snprintf(g_lock_file, sizeof(g_lock_file), LCFILE_PATH_FMT, user);
 
+    NSApplicationLoad();
     g_pid = getpid();
+    g_event_bytes = malloc(0xf8);
     g_connection = SLSMainConnectionID();
     g_cv_host_clock_frequency   = CVGetHostClockFrequency();
     g_layer_normal_window_level = CGWindowLevelForKey(LAYER_NORMAL);
@@ -184,12 +146,12 @@ static inline bool configure_settings_and_acquire_lock(void)
     g_layer_above_window_level  = CGWindowLevelForKey(LAYER_ABOVE);
     CGSGetConnectionPortById    = macho_find_symbol("/System/Library/PrivateFrameworks/SkyLight.framework/Versions/A/SkyLight", "_CGSGetConnectionPortById");
 
-    NSApplicationLoad();
     signal(SIGCHLD, SIG_IGN);
     signal(SIGPIPE, SIG_IGN);
     CGSetLocalEventsSuppressionInterval(0.0f);
     CGEnableEventStateCombining(false);
     mouse_state_init(&g_mouse_state);
+    task_get_special_port(mach_task_self(), TASK_BOOTSTRAP_PORT, &g_bs_port);
 
 #if 0
     hook_nsobject_autorelease();
@@ -215,29 +177,6 @@ static inline bool configure_settings_and_acquire_lock(void)
 }
 #pragma clang diagnostic pop
 
-__attribute__((no_sanitize("undefined")))
-static CONNECTION_CALLBACK(connection_handler)
-{
-    //
-    // NOTE(koekeishiya): Disable undefined sanitizer for this particular function.
-    // It will sometimes report load of misaligned address when reading from the
-    // data buffer, but there is nothing for us to do here because said memory is
-    // allocated and managed by macOS.
-    //
-
-    if (type == 1204) {
-        event_loop_post(&g_event_loop, MISSION_CONTROL_ENTER, NULL, 0);
-    } else if (type == 1325) {
-        event_loop_post(&g_event_loop, SLS_SPACE_ADD_WINDOW, (void *) (intptr_t) (* (uint64_t *) data), (int) (* (uint32_t *) (data + 8)));
-    } else if (type == 1326) {
-        event_loop_post(&g_event_loop, SLS_SPACE_REMOVE_WINDOW, (void *) (intptr_t) (* (uint64_t *) data), (int) (* (uint32_t *) (data + 8)));
-    } else if (type == 1327) {
-        event_loop_post(&g_event_loop, SLS_SPACE_CREATED, (void *) (intptr_t) (* (uint64_t *) data), 0);
-    } else if (type == 1328) {
-        event_loop_post(&g_event_loop, SLS_SPACE_DESTROYED, (void *) (intptr_t) (* (uint64_t *) data), 0);
-    }
-}
-
 static void parse_arguments(int argc, char **argv)
 {
     if ((string_equals(argv[1], HELP_OPT_LONG)) ||
@@ -317,6 +256,7 @@ static void parse_arguments(int argc, char **argv)
     }
 }
 
+#ifndef TESTS
 int main(int argc, char **argv)
 {
     if (argc > 1) {
@@ -335,11 +275,11 @@ int main(int argc, char **argv)
         require("yabai: 'display has separate spaces' is disabled! abort..\n");
     }
 
-    if (!ts_init(MEGABYTES(4))) {
+    if (!ts_init(MEGABYTES(8))) {
         error("yabai: could not allocate temporary storage! abort..\n");
     }
 
-    if (!memory_pool_init(&g_signal_storage, KILOBYTES(128))) {
+    if (!memory_pool_init(&g_signal_storage, KILOBYTES(256))) {
         error("yabai: could not allocate event signal storage! abort..\n");
     }
 
@@ -369,16 +309,16 @@ int main(int argc, char **argv)
 
     if (workspace_is_macos_monterey() || workspace_is_macos_ventura() || workspace_is_macos_sonoma()) {
         mission_control_observe();
+
+        if (workspace_is_macos_ventura() || workspace_is_macos_sonoma()) {
+            SLSRegisterConnectionNotifyProc(g_connection, connection_handler, 1327, NULL);
+            SLSRegisterConnectionNotifyProc(g_connection, connection_handler, 1328, NULL);
+        }
     } else {
         SLSRegisterConnectionNotifyProc(g_connection, connection_handler, 1204, NULL);
     }
 
-    if (workspace_is_macos_ventura() || workspace_is_macos_sonoma()) {
-        SLSRegisterConnectionNotifyProc(g_connection, connection_handler, 1325, NULL);
-        SLSRegisterConnectionNotifyProc(g_connection, connection_handler, 1326, NULL);
-        SLSRegisterConnectionNotifyProc(g_connection, connection_handler, 1327, NULL);
-        SLSRegisterConnectionNotifyProc(g_connection, connection_handler, 1328, NULL);
-    }
+    SLSRegisterConnectionNotifyProc(g_connection, connection_handler, 808, NULL);
 
     window_manager_init(&g_window_manager);
     space_manager_begin(&g_space_manager);
@@ -388,7 +328,7 @@ int main(int argc, char **argv)
         error("yabai: could not start message loop! abort..\n");
     }
 
-    exec_config_file();
+    exec_config_file(g_config_file, sizeof(g_config_file));
 
     for (;;) {
         NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
@@ -400,3 +340,6 @@ int main(int argc, char **argv)
 
     return 0;
 }
+#endif
+
+PROFILER_END_TRANSLATION_UNIT;
